class _StringTable:
    # CSV Header Strings
    ClientId = "Client Id"
    Type = "Type"
    Status = "Status"
    Campaign = "Campaign"
    Id = "Id"
    BusinessId = "Business Id"
    ParentId = "Parent Id"
    CampaignId = "Campaign Id"
    TimeZone = "Time Zone"
    Budget = "Budget"
    BudgetType = "Budget Type"
    BudgetName = "Budget Name"
    Experiment = "Experiment"
    TrafficSplitPercent = "Traffic Split Percent"
    BaseCampaignId = "Base Campaign Id"
    ExperimentCampaignId = "Experiment Campaign Id"
    ExperimentId = "Experiment Id"
    FeedId = "Feed Id"
    ExperimentType = "Experiment Type"
    BudgetId = "Budget Id"
    AdGroup = "Ad Group"
    AdGroupType = "Ad Group Type"
    HotelAdGroupType = "Hotel Ad Group Type"
    CommissionRate = "Commission Rate"
    PercentCpcBid = "Percent Cpc Bid"
    FinalUrlExpansionOptOut = "Url Expansion Opt Out"
    HotelListingGroupType = "Ad Group Hotel Listing Group"
    HotelAttribute = "Hotel Attribute"
    HotelAttributeValue = "Hotel Attribute Value"
    Keyword = "Keyword"
    TextAd = "Text Ad"
    ProductAd = "Product Ad"
    AppInstallAd = "App Install Ad"
    DynamicSearchAd = "Dynamic Search Ad"
    ExpandedTextAd = "Expanded Text Ad"
    Title = "Title"
    EditorialStatus = "Editorial Status"
    EditorialAppealStatus = "Editorial Appeal Status"
    Error = "Error"
    ErrorNumber = "Error Number"
    EditorialLocation = "Editorial Location"
    EditorialTerm = "Editorial Term"
    EditorialReasonCode = "Editorial Reason Code"
    MigrationErrors = "Migration Errors"
    DisplayUrl = "Display Url"
    DestinationUrl = "Destination Url"
    BusinessName = "Business Name"
    PhoneNumber = "Phone Number"
    PromotionalText = "Promotion"
    MatchType = "Match Type"
    Param1 = "Param1"
    Param2 = "Param2"
    Param3 = "Param3"
    DevicePreference = "Device Preference"
    Details = "Verified Tracking Setting"
    CampaignNegativeKeyword = "Campaign Negative Keyword"
    CampaignNegativeSite = "Campaign Negative Site"
    AdGroupNegativeKeyword = "Ad Group Negative Keyword"
    KeywordNegativeKeyword = "Keyword Negative Keyword"
    AdGroupNegativeSite = "Ad Group Negative Site"
    Text = "Text"
    TextPart2="Text Part 2"
    Website = "Website"
    Target = "Target"
    Feed = "Feed"
    FeedItem = "Feed Item"
    FeedName = "Feed Name"
    TargetAdGroupId = 'Target Ad Group Id'
    TargetCampaignId = 'Target Campaign Id'
    CustomAttributes = "Custom Attributes"
    PageFeedIds = "Page Feed Ids"
    PhysicalIntent = "Physical Intent"
    Bid = "Bid"
    Profile = "Profile"
    ProfileId = "Profile Id"
    BidAdjustment = "Bid Adjustment"
    CashbackAdjustment = "Cashback Percent"
    SubType = "Sub Type"
    OsNames = "OS Names"
    StartDate = "Start Date"
    EndDate = "End Date"
    NetworkDistribution = "Network Distribution"
    Language = "Language"
    CpcBid = "Cpc Bid"
    CpvBid = "Cpv Bid"
    CpmBid = "Cpm Bid"
    AdRotation = "Ad Rotation"
    PrivacyStatus = "Privacy Status"
    Account = "Account"
    SyncTime = "Sync Time"
    Name = "Name"
    MSCLKIDAutoTaggingEnabled = "MSCLKID Auto Tagging Enabled"
    LastModifiedTime = "Modified Time"
    AdFormatPreference = "Ad Format Preference"
    IncludeViewThroughConversions = "Include View Through Conversions"
    ProfileExpansionEnabled = "Profile Expansion Enabled"
    AdClickParallelTracking = "Ad Click Parallel Tracking"
    AutoApplyRecommendations = "Auto Apply Recommendations"
    AllowImageAutoRetrieve = "Allow Image Auto Retrieve"
    MultiMediaAdBidAdjustment = "Multi Media Ad Bid Adjustment"
    UseOptimizedTargeting = "Use Optimized Targeting"
    BusinessAttributes = "Business Attributes"
    Schedule = "Schedule"

    # Entity Types
    SemanticVersion = "Format Version"
    LocationTarget = "Location Target"
    RadiusTarget = "Radius Target"
    BusinessTarget = "Business Location Target"
    RadiusTargetId = "Radius Target Id"
    NegativeLocationTarget = "Negative Location Target"
    AgeTarget = "Age Target"
    GenderTarget = "Gender Target"
    DeviceOsTarget = "DeviceOS Target"
    Radius = "Radius"
    Unit = "Unit"
    TargetAll = "Target All"
    LocationAdExtension = "Location Ad Extension"
    CallAdExtension = "Call Ad Extension"
    ImageAdExtension = "Image Ad Extension"
    SocialNetworkAdExtension = "Social Network Ad Extension"
    Version = "Version"

    # Disclaimer Ads
    DisclaimerAdsEnabled = "Disclaimer Ads Enabled"
    DisclaimerName = "Disclaimer Name"
    DisclaimerTitle = "Disclaimer Title"
    DisclaimerLayout = "Disclaimer Layout"
    DisclaimerPopupText = "Disclaimer Popup Text"
    DisclaimerLineText = "Disclaimer Line Text"

    DisclaimerAdExtension = "Disclaimer Ad Extension"
    CampaignDisclaimerAdExtension = "Campaign Disclaimer Ad Extension"

    # Image Ad Extension
    AltText = "Alternative Text"
    MediaIds = "Media Ids"
    AccountImageAdExtension = "Account Image Ad Extension"
    CampaignImageAdExtension = "Campaign Image Ad Extension"
    AdGroupImageAdExtension = "Ad Group Image Ad Extension"
    Layouts = "Layouts"
    DisplayText = "Display Text"

    # Filter Link ad extension
    FilterLinkAdExtension = "Filter Link Ad Extension"
    AccountFilterLinkAdExtension = "Account Filter Link Ad Extension"
    CampaignFilterLinkAdExtension = "Campaign Filter Link Ad Extension"
    AdGroupFilterLinkAdExtension = "Ad Group Filter Link Ad Extension"
    AdExtensionHeaderType = "AdExtension Header Type"
    Texts = "Texts"

    SiteLinkExtensionOrder = "Sitelink Extension Order"
    SiteLinkDisplayText = "Sitelink Extension Link Text"
    SiteLinkDestinationUrl = "Sitelink Extension Destination Url"
    SiteLinkDescription1 = "Sitelink Extension Description1"
    SiteLinkDescription2 = "Sitelink Extension Description2"

    # Location Ad Extensions
    AddressLine1 = "Address Line 1"
    AddressLine2 = "Address Line 2"
    PostalCode = "Postal Code"
    City = "City"
    StateOrProvince = "State Or Province Code"
    ProvinceName = "Province Name"
    GeoCodeStatus = "Geo Code Status"
    ImageMediaId = "Business Icon"
    AccountLocationAdExtension = "Account Location Ad Extension"
    CampaignLocationAdExtension = "Campaign Location Ad Extension"

    # Call Ad Extensions
    CountryCode = "Country Code"
    IsCallOnly = "Call Only"
    CampaignCallAdExtension = "Campaign Call Ad Extension"
    IsCallTrackingEnabled = "Call Tracking Enabled"
    RequireTollFreeTrackingNumber = "Toll Free"

    #Action Ad Extension
    ActionAdExtension = "Action Ad Extension"
    AccountActionAdExtension = "Account Action Ad Extension"
    CampaignActionAdExtension = "Campaign Action Ad Extension"
    AdGroupActionAdExtension = "Ad Group Action Ad Extension"
    ActionType = "Action Type"
    ActionText = "Action Text"

    #Video Ad Extension
    VideoAdExtension = "Video Ad Extension"
    ThumbnailUrl = "Thumbnail Url"
    ThumbnailId = "Thumbnail Id"
    VideoId = "Video Id"
    AccountVideoAdExtension = "Account Video Ad Extension"
    CampaignVideoAdExtension = "Campaign Video Ad Extension"
    AdGroupVideoAdExtension = "Ad Group Video Ad Extension"

    #Promotion AdExtension
    PromotionAdExtension = "Promotion Ad Extension"
    AccountPromotionAdExtension = "Account Promotion Ad Extension"
    CampaignPromotionAdExtension = "Campaign Promotion Ad Extension"
    AdGroupPromotionAdExtension = "Ad Group Promotion Ad Extension"
    PromotionTarget = "Promotion Target"
    DiscountModifier = "Discount Modifier"
    PercentOff = "Percent Off"
    MoneyAmountOff = "Money Amount Off"
    PromotionCode = "Promotion Code"
    OrdersOverAmount = "Orders Over Amount"
    Occasion = "Occasion"
    PromotionStart = "Promotion Start"
    PromotionEnd = "Promotion End"
    CurrencyCode = "Currency Code"

    # Structured Snippet Ad Extension
    StructuredSnippetAdExtension = "Structured Snippet Ad Extension"
    AccountStructuredSnippetAdExtension = "Account Structured Snippet Ad Extension"
    CampaignStructuredSnippetAdExtension = "Campaign Structured Snippet Ad Extension"
    AdGroupStructuredSnippetAdExtension = "Ad Group Structured Snippet Ad Extension"
    StructuredSnippetHeader = "Structured Snippet Header"
    StructuredSnippetValues = "Structured Snippet Values"

    # Sitelink Ad Extension
    SitelinkAdExtension = "Sitelink Ad Extension"
    AdGroupSitelinkAdExtension = "Ad Group Sitelink Ad Extension"
    CampaignSitelinkAdExtension = "Campaign Sitelink Ad Extension"
    AccountSitelinkAdExtension = "Account Sitelink Ad Extension"


    # Flyer Ad Extension
    FlyerAdExtension = "Flyer Ad Extension"
    AccountFlyerAdExtension = "Account Flyer Ad Extension"
    CampaignFlyerAdExtension = "Campaign Flyer Ad Extension"
    AdGroupFlyerAdExtension = "Ad Group Flyer Ad Extension"
    FlyerName = "Flyer Name"
    MediaUrls = "Media Urls"


    # Editorial Rejection Reasons
    PublisherCountries = "Publisher Countries"

    # BTE Types
    KeywordFirstPageBidType = "Keyword First Page Bid"
    KeywordMainLineBidType = "Keyword Main Line Bid"
    KeywordBestPositionBidType = "Keyword Best Position Bid"

    ProductCondition1 = "Product Condition 1"
    ProductCondition2 = "Product Condition 2"
    ProductCondition3 = "Product Condition 3"
    ProductCondition4 = "Product Condition 4"
    ProductCondition5 = "Product Condition 5"
    ProductCondition6 = "Product Condition 6"
    ProductCondition7 = "Product Condition 7"
    ProductCondition8 = "Product Condition 8"
    ProductValue1 = "Product Value 1"
    ProductValue2 = "Product Value 2"
    ProductValue3 = "Product Value 3"
    ProductValue4 = "Product Value 4"
    ProductValue5 = "Product Value 5"
    ProductValue6 = "Product Value 6"
    ProductValue7 = "Product Value 7"
    ProductValue8 = "Product Value 8"

    MerchantCenterId = "Store Id"
    MerchantCenterName = "Store Name"

    # App Ad Extension
    AppAdExtension = "App Ad Extension"
    AppPlatform = "App Platform"
    AppStoreId = "App Id"
    AccountAppAdExtension = "Account App Ad Extension"
    CampaignAppAdExtension = "Campaign App Ad Extension"
    AdGroupAppAdExtension = "Ad Group App Ad Extension"
    IsTrackingEnabled = "Tracking Enabled"

    # Misc Stuff
    EntityActiveStatus = "Active"
    EntityDeletedStatus = "Deleted"
    IntraFieldSeparator = ';'
    Active = "Active"
    Latitude = "Latitude"
    Longitude = "Longitude"

    # BI
    Spend = "Spend"
    Impressions = "Impressions"
    Clicks = "Clicks"
    CTR = "CTR"
    AvgCPC = "Avg CPC"
    AvgCPM = "Avg CPM"
    AvgPosition = "Avg position"
    Conversions = "Conversions"
    CPA = "CPA"
    QualityScore = "Quality Score"
    KeywordRelevance = "Keyword Relevance"
    LandingPageRelevance = "Landing Page Relevance"
    LandingPageUserExperience = "Landing Page User Experience"

    # DayTime Criterion
    FromHour = "From Hour"
    ToHour = "To Hour"
    FromMinute = "From Minute"
    ToMinute = "To Minute"

    # Shared Entities
    NegativeKeywordList = "Negative Keyword List"
    CampaignNegativeKeywordList = "Campaign Negative Keyword List Association"
    ListNegativeKeyword = "Shared Negative Keyword"

    # Subtypes
    MetroAreaSubType = "Metro Area"
    CountrySubType = "Country"
    StateSubType = "State"
    CitySubType = "City"
    PostalCodeSubType = "Postal Code"

    # Bing Shopping Campaigns
    IsExcluded = "Is Excluded"
    ParentAdGroupCriterionId = "Parent Criterion Id"
    CampaignType = "Campaign Type"
    CampaignPriority = "Priority"
    LocalInventoryAdsEnabled = "LocalInventoryAdsEnabled"
    DynamicDescriptionEnabled = "Dynamic Description Enabled"

    CampaignNegativeStoreCriterion = "Campaign Negative Store Criterion"


    # CoOp
    BidOption = "Bid Option"
    BidBoostValue = "Bid Boost Value"
    MaximumBid = "Maximum Bid"

    # V10 added
    FieldPath = "Field Path"

    # Upgrade URL
    FinalUrl = "Final Url"
    FinalMobileUrl = "Mobile Final Url"
    TrackingTemplate = "Tracking Template"
    CustomParameter = "Custom Parameter"

    # Review Ad Extension
    ReviewAdExtension = "Review Ad Extension"
    AccountReviewAdExtension = "Account Review Ad Extension"
    CampaignReviewAdExtension = "Campaign Review Ad Extension"
    AdGroupReviewAdExtension = "Ad Group Review Ad Extension"
    IsExact = "Is Exact"
    Source = "Source"
    Url = "Url"

    # Price Ad Extension
    PriceAdExtension = "Price Ad Extension"
    AccountPriceAdExtension = "Account Price Ad Extension"
    CampaignPriceAdExtension = "Campaign Price Ad Extension"
    AdGroupPriceAdExtension = "Ad Group Price Ad Extension"
    PriceExtensionType = "Price Extension Type"
    Header1 = "Header 1"
    Header2 = "Header 2"
    Header3 = "Header 3"
    Header4 = "Header 4"
    Header5 = "Header 5"
    Header6 = "Header 6"
    Header7 = "Header 7"
    Header8 = "Header 8"
    PriceDescription1 = "Price Description 1"
    PriceDescription2 = "Price Description 2"
    PriceDescription3 = "Price Description 3"
    PriceDescription4 = "Price Description 4"
    PriceDescription5 = "Price Description 5"
    PriceDescription6 = "Price Description 6"
    PriceDescription7 = "Price Description 7"
    PriceDescription8 = "Price Description 8"
    FinalUrl1 = "Final Url 1"
    FinalUrl2 = "Final Url 2"
    FinalUrl3 = "Final Url 3"
    FinalUrl4 = "Final Url 4"
    FinalUrl5 = "Final Url 5"
    FinalUrl6 = "Final Url 6"
    FinalUrl7 = "Final Url 7"
    FinalUrl8 = "Final Url 8"
    FinalMobileUrl1 = "Final Mobile Url 1"
    FinalMobileUrl2 = "Final Mobile Url 2"
    FinalMobileUrl3 = "Final Mobile Url 3"
    FinalMobileUrl4 = "Final Mobile Url 4"
    FinalMobileUrl5 = "Final Mobile Url 5"
    FinalMobileUrl6 = "Final Mobile Url 6"
    FinalMobileUrl7 = "Final Mobile Url 7"
    FinalMobileUrl8 = "Final Mobile Url 8"
    Price1 = "Price 1"
    Price2 = "Price 2"
    Price3 = "Price 3"
    Price4 = "Price 4"
    Price5 = "Price 5"
    Price6 = "Price 6"
    Price7 = "Price 7"
    Price8 = "Price 8"
    CurrencyCode1 = "Currency Code 1"
    CurrencyCode2 = "Currency Code 2"
    CurrencyCode3 = "Currency Code 3"
    CurrencyCode4 = "Currency Code 4"
    CurrencyCode5 = "Currency Code 5"
    CurrencyCode6 = "Currency Code 6"
    CurrencyCode7 = "Currency Code 7"
    CurrencyCode8 = "Currency Code 8"
    PriceUnit1 = "Price Unit 1"
    PriceUnit2 = "Price Unit 2"
    PriceUnit3 = "Price Unit 3"
    PriceUnit4 = "Price Unit 4"
    PriceUnit5 = "Price Unit 5"
    PriceUnit6 = "Price Unit 6"
    PriceUnit7 = "Price Unit 7"
    PriceUnit8 = "Price Unit 8"
    PriceQualifier1 = "Price Qualifier 1"
    PriceQualifier2 = "Price Qualifier 2"
    PriceQualifier3 = "Price Qualifier 3"
    PriceQualifier4 = "Price Qualifier 4"
    PriceQualifier5 = "Price Qualifier 5"
    PriceQualifier6 = "Price Qualifier 6"
    PriceQualifier7 = "Price Qualifier 7"
    PriceQualifier8 = "Price Qualifier 8"

    # Callout Ad Extension
    CalloutAdExtension = "Callout Ad Extension"
    AccountCalloutAdExtension = "Account Callout Ad Extension"
    CampaignCalloutAdExtension = "Campaign Callout Ad Extension"
    AdGroupCalloutAdExtension = "Ad Group Callout Ad Extension"
    CalloutText = "Callout Text"

    # Bid Strategy
    BidStrategy = "Bid Strategy"
    BidStrategyId = "Bid Strategy Id"
    BidStrategyName = "Bid Strategy Name"
    BidStrategyType = "Bid Strategy Type"
    BidStrategyMaxCpc = "Bid Strategy MaxCpc"
    BidStrategyTargetCpa = "Bid Strategy TargetCpa"
    BidStrategyTargetRoas = "Bid Strategy TargetRoas"
    InheritedBidStrategyType = "Inherited Bid Strategy Type"
    BidStrategyTargetAdPosition = "Bid Strategy TargetAdPosition"
    BidStrategyTargetImpressionShare = "Bid Strategy TargetImpressionShare"
    BidStrategyPercentMaxCpc = "Bid Strategy PercentMaxCpc"
    BidStrategyCommissionRate = "Bid Strategy CommissionRate"
    BidStrategyTargetCostPerSale = "Bid Strategy TargetCostPerSale"

    # Remarketing
    Audience = "Audience"
    Description = "Description"
    MembershipDuration = "Membership Duration"
    Scope = "Scope"
    TagId = "UET Tag Id"
    AudienceId = "Audience Id"
    TargetSetting = "Target Setting"
    RemarketingRule = "Remarketing Rule"
    AudienceSearchSize = "Audience Search Size"
    ProductAudience = "Product Audience"
    AudienceNetworkSize = "Audience Network Size"
    SupportedCampaignTypes = "Supported Campaign Types"
    ProductAudienceType = "Product Audience Type"
    CombinationRule = "Combination Rule"
    SourceId = "Source Id"
    AdGroupProductAudienceAssociation = "Ad Group Product Audience Association"
    AdGroupNegativeProductAudienceAssociation = "Ad Group Negative Product Audience Association"
    CampaignProductAudienceAssociation = "Campaign Product Audience Association"
    CampaignNegativeProductAudienceAssociation = "Campaign Negative Product Audience Association"
    SimilarRemarketingList = "Similar Remarketing List"
    AdGroupSimilarRemarketingListAssociation = "Ad Group Similar Remarketing List Association"
    AdGroupNegativeSimilarRemarketingListAssociation = "Ad Group Negative Similar Remarketing List Association"
    CampaignSimilarRemarketingListAssociation = "Campaign Similar Remarketing List Association"
    CampaignNegativeSimilarRemarketingListAssociation = "Campaign Negative Similar Remarketing List Association"
    RemarketingList = "Remarketing List"
    AdGroupRemarketingListAssociation = 'Ad Group Remarketing List Association'
    AdGroupNegativeRemarketingListAssociation = 'Ad Group Negative Remarketing List Association'
    CampaignRemarketingListAssociation = 'Campaign Remarketing List Association'
    CampaignNegativeRemarketingListAssociation = 'Campaign Negative Remarketing List Association'
    CustomAudience = "Custom Audience"
    AdGroupCustomAudienceAssociation = "Ad Group Custom Audience Association"
    AdGroupNegativeCustomAudienceAssociation = "Ad Group Negative Custom Audience Association"
    CampaignCustomAudienceAssociation = "Campaign Custom Audience Association"
    CampaignNegativeCustomAudienceAssociation = "Campaign Negative Custom Audience Association"
    InMarketAudience = "In Market Audience"
    AdGroupInMarketAudienceAssociation = "Ad Group In Market Audience Association"
    AdGroupNegativeInMarketAudienceAssociation = "Ad Group Negative In Market Audience Association"
    CampaignInMarketAudienceAssociation = "Campaign In Market Audience Association"
    CampaignNegativeInMarketAudienceAssociation = "Campaign Negative In Market Audience Association"
    CombinedList = "Combined List"
    AdGroupCombinedListAssociation = "Ad Group Combined List Association"
    AdGroupNegativeCombinedListAssociation = "Ad Group Negative Combined List Association"
    CampaignCombinedListAssociation = "Campaign Combined List Association"
    CampaignNegativeCombinedListAssociation = "Campaign Negative Combined List Association"
    CustomerList = "Customer List"
    CustomerListItem = "Customer List Item"
    AdGroupCustomerListAssociation = "Ad Group Customer List Association"
    AdGroupNegativeCustomerListAssociation = "Ad Group Negative Customer List Association"
    CampaignCustomerListAssociation = "Campaign Customer List Association"
    CampaignNegativeCustomerListAssociation = "Campaign Negative Customer List Association"
    ImpressionBasedRemarketingList = "Impression Based Remarketing List"
    AdGroupImpressionBasedRemarketingListAssociation = "Ad Group Impression Based Remarketing List Association"
    AdGroupNegativeImpressionBasedRemarketingListAssociation = "Ad Group Negative Impression Based Remarketing List Association"
    CampaignImpressionBasedRemarketingListAssociation = "Campaign Impression Based Remarketing List Association"
    CampaignNegativeImpressionBasedRemarketingListAssociation = "Campaign Negative Impression Based Remarketing List Association"
    EntityType = "Entity Type"
    ImpressionCampaignId = "Impression Campaign Id"
    ImpressionAdGroupId = "Impression Ad Group Id"

    # Expanded Text Ad
    TitlePart1 = "Title Part 1"
    TitlePart2 = "Title Part 2"
    TitlePart3 = "Title Part 3"
    Path1 = "Path 1"
    Path2 = "Path 2"
    Domain = "Domain"

    # Ad Extension Scheduling
    AdSchedule = "Ad Schedule"

    #UseSearcherTimeZone
    UseSearcherTimeZone = "Use Searcher Time Zone"
    AdScheduleUseSearcherTimeZone = "Ad Schedule Use Searcher Time Zone"

    # Dynamic Search Ads
    DomainLanguage = "Domain Language"
    DynamicAdTargetCondition1 = "Dynamic Ad Target Condition 1"
    DynamicAdTargetValue1 = "Dynamic Ad Target Value 1"
    DynamicAdTargetCondition2 = "Dynamic Ad Target Condition 2"
    DynamicAdTargetValue2 = "Dynamic Ad Target Value 2"
    DynamicAdTargetCondition3 = "Dynamic Ad Target Condition 3"
    DynamicAdTargetValue3 = "Dynamic Ad Target Value 3"
    DynamicAdTargetConditionOperator1 = "Dynamic Ad Target Condition Operator 1"
    DynamicAdTargetConditionOperator2 = "Dynamic Ad Target Condition Operator 2"
    DynamicAdTargetConditionOperator3 = "Dynamic Ad Target Condition Operator 3"

    AdGroupAgeCriterion = "Ad Group Age Criterion"
    AdGroupDayTimeCriterion = "Ad Group DayTime Criterion"
    AdGroupDeviceCriterion = "Ad Group DeviceOS Criterion"
    AdGroupGenderCriterion = "Ad Group Gender Criterion"
    AdGroupRadiusCriterion = "Ad Group Radius Criterion"
    AdGroupLocationCriterion = "Ad Group Location Criterion"
    AdGroupLocationIntentCriterion = "Ad Group Location Intent Criterion"
    AdGroupNegativeLocationCriterion = "Ad Group Negative Location Criterion"
    AdGroupCompanyNameCriterion = "Ad Group Company Name Criterion"
    AdGroupIndustryCriterion = "Ad Group Industry Criterion"
    AdGroupJobFunctionCriterion = "Ad Group Job Function Criterion"
    AdGroupNegativeAgeCriterion = "Ad Group Negative Age Criterion"
    AdGroupNegativeCompanyNameCriterion = "Ad Group Negative Company Name Criterion"
    AdGroupNegativeGenderCriterion = "Ad Group Negative Gender Criterion"
    AdGroupNegativeIndustryCriterion = "Ad Group Negative Industry Criterion"
    AdGroupNegativeJobFunctionCriterion = "Ad Group Negative Job Function Criterion"
    AdGroupGenreCriterion = "Ad Group Genre Criterion"

    # Responsive Ad
    ResponsiveAd = "Responsive Ad"
    CallToAction = "Call To Action"
    Headline = "Headline"
    Images = "Images"
    LandscapeImageMediaId = "Landscape Image Media Id"
    LandscapeLogoMediaId = "Landscape Logo Media Id"
    LongHeadline = "Long Headline"
    LongHeadlines = "Long Headlines"
    SquareImageMediaId = "Square Image Media Id"
    SquareLogoMediaId = "Square Logo Media Id"
    ImpressionTrackingUrls = "Impression Tracking Urls"
    CallToActionLanguage = "Call To Action Language"
    Descriptions = "Descriptions"
    Headlines = "Headlines"
    Videos = "Videos"

    #Image
    Image = "Image"
    Height = "Height"
    Width = "Width"

    # Video
    Video = "Video"
    SourceUrl = "Source Url"
    AspectRatio = "Aspect Ratio"
    DurationInMillionSeconds = "Duration In Milliseconds"

    # Responsive Search Ad
    ResponsiveSearchAd = "Responsive Search Ad"

    # Labels
    ColorCode = "Color"
    Label = "Label"
    CampaignLabel = "Campaign Label"
    AdGroupLabel = "Ad Group Label"
    KeywordLabel = "Keyword Label"
    AppInstallAdLabel = "App Install Ad Label"
    DynamicSearchAdLabel = "Dynamic Search Ad Label"
    ExpandedTextAdLabel = "Expanded Text Ad Label"
    ProductAdLabel = "Product Ad Label"
    TextAdLabel = "Text Ad Label"
    ResponsiveAdLabel = "Responsive Ad Label"
    ResponsiveSearchAdLabel = "Responsive Search Ad Label"

    # Offline Conversions
    OfflineConversion = "Offline Conversion"
    ConversionCurrencyCode = "Conversion Currency Code"
    ConversionName = "Conversion Name"
    ConversionTime = "Conversion Time"
    ConversionValue = "Conversion Value"
    MicrosoftClickId = "Microsoft Click Id"
    AdjustmentValue = "Adjustment Value"
    AdjustmentTime = "Adjustment Time"
    AdjustmentCurrencyCode = "Adjustment Currency Code"
    AdjustmentType = "Adjustment Type"
    ExternalAttributionModel = "External Attribution Model"
    ExternalAttributionCredit = "External Attribution Credit"
    HashedEmailAddress = "Hashed Email Address"
    HashedPhoneNumber = "Hashed Phone Number"

    # Campaign Criterion
    CampaignAgeCriterion = 'Campaign Age Criterion'
    CampaignDayTimeCriterion = 'Campaign DayTime Criterion'
    CampaignDeviceOSCriterion = 'Campaign DeviceOS Criterion'
    CampaignGenderCriterion = 'Campaign Gender Criterion'
    CampaignLocationCriterion = 'Campaign Location Criterion'
    CampaignLocationIntentCriterion = 'Campaign Location Intent Criterion'
    CampaignNegativeLocationCriterion = 'Campaign Negative Location Criterion'
    CampaignRadiusCriterion = 'Campaign Radius Criterion'
    CampaignCompanyNameCriterion = 'Campaign Company Name Criterion'
    CampaignJobFunctionCriterion = 'Campaign Job Function Criterion'
    CampaignIndustryCriterion = 'Campaign Industry Criterion'
    CampaignDealCriterion = 'Campaign Deal Criterion'

    # Online Conversion
    OnlineConversionAdjustment = "Online Conversion Adjustment"
    TransactionId = "Transaction Id"

    # Hotel Ad
    AdGroupAdvanceBookingWindowCriterion = "Ad Group Advance Booking Window Criterion"
    AdGroupCheckInDayCriterion = "Ad Group Check In Day Criterion"
    AdGroupLengthOfStayCriterion = "Ad Group Length of Stay Criterion"
    AdGroupHotelDateSelectionTypeCriterion = "Ad Group Hotel Date Selection Type Criterion"
    AdGroupCheckInDateCriterion = "Ad Group Check In Date Criterion"
    MinTargetValue = "Min Target Value"
    MaxTargetValue = "Max Target Value"

    FinalUrlSuffix = "Final Url Suffix"

    # AdCustomizerAttribute
    AdCustomizerAttribute = "Adcustomizer Attribute"
    AdCustomizerCampaign = "Campaign Adcustomizer Attribute"
    AdCustomizerAdGroup = "Adgroup Adcustomizer Attribute"
    AdCustomizerKeyword = "Keyword Adcustomizer Attribute"
    AdCustomizerDataType = "AdCustomizer DataType"
    AdCustomizerAttributeValue = "AdCustomizer AttributeValue"

    # Campaign Conversion Goal
    CampaignConversionGoal = "Campaign Conversion Goal"
    GoalId = "Goal Id"

    # PMax
    AssetGroup = "Asset Group"
    AudienceGroup = "Audience Group"
    CampaignNegativeWebpage = "Campaign Negative Webpage"
    AssetGroupListingGroup = "Asset Group Listing Group"
    AudienceGroupAssetGroupAssociation = "Audience Group Asset Group Association"
    Audiences = "Audiences"
    AudienceGroupName = "Audience Group Name"
    AgeRanges = "Age Ranges"
    GenderTypes = "Gender Types"
    ParentListingGroupId = "Parent Listing Group Id"
    AutoGeneratedTextOptOut = "Auto Generated Text Assets Opt Out"
    AutoGeneratedImageOptOut = "Auto Generated Image Assets Opt Out"
    CostPerSaleOptOut = "Cost Per Sale Opt Out"
    AssetGroupSearchTheme = "Asset Group Search Theme"
    SearchTheme = "Search Theme"

    # MultiChannel Campaign
    DestinationChannel = "Destination Channel"
    IsMultiChannelCampaign = "Is Multi Channel Campaign"

    # Seasonality Adjustment
    SeasonalityAdjustment = "Seasonality Adjustment"
    DataExclusion = "Data Exclusion"
    DeviceType = "Device Type"
    CampaignAssociations = "Campaign Associations"

    # DNV Serving on MSAN
    ShouldServeOnMSAN = "Should Serve On MSAN"

    # Conversion Goal
    AttributionModelType = "Attribution Model Type"
    ConversionWindowInMinutes = "Conversion Window In Minutes"
    CountType = "Count Type"
    ExcludeFromBidding = "Exclude From Bidding"
    GoalCategory = "Goal Category"
    IsEnhancedConversionsEnabled = "Is Enhanced Conversions Enabled"
    RevenueType = "Revenue Type"
    RevenueValue = "Revenue Value"
    TrackingStatus = "Tracking Status"
    ViewThroughConversionWindowInMinutes = "View Through Conversion Window In Minutes"
    MinimumDurationInSecond = "Minimum Duration In Second"
    ActionExpression = "Action Expression"
    ActionOperator = "Action Operator"
    CategoryExpression = "Category Expression"
    CategoryOperator = "Category Operator"
    LabelExpression = "Label Expression"
    LabelOperator = "Label Operator"
    EventValue = "Event Value"
    EventValueOperator = "Event Value Operator"
    IsExternallyAttributed = "Is Externally Attributed"
    MinimumPagesViewed = "Minimum Pages Viewed"
    UrlExpression = "URL Expression"
    UrlOperator = "URL Operator"
    ConversionGoal = "Conversion Goal"
    EventGoal = "Event Goal"
    AppInstallGoal = "AppInstall Goal"
    MultiStageGoal = "MultiStage Goal"
    DurationGoal = "Duration Goal"
    OfflineConversionGoal = "OfflineConversion Goal"
    UrlGoal = "URL Goal"
    InStoreTransactionGoal = "InStoreTransaction Goal"
    PagesViewedPerVisitGoal = "PagesViewedPerVisit Goal"
    SmartGoal = "Smart Goal"
    InStoreVisitGoal = "InStoreVisit Goal"
    ProductGoal = "Product Goal"

    # Brand List
    BrandList = "Brand List"
    BrandItem = "Brand Item"
    CampaignBrandList = "Campaign Brand List Association"
    BrandId = "Brand Id"
    BrandUrl = "Brand Url"
    BrandName = "Brand Name"
    StatusDateTime = "Editorial Status Date"

    # Asset Group Url Target
    AssetGroupUrlTarget = "Asset Group Url Target";
    AssetGroupTargetCondition1 = "Asset Group Target Condition 1";
    AssetGroupTargetCondition2 = "Asset Group Target Condition 2";
    AssetGroupTargetCondition3 = "Asset Group Target Condition 3";
    AssetGroupTargetConditionOperator1 = "Asset Group Target Condition Operator 1";
    AssetGroupTargetConditionOperator2 = "Asset Group Target Condition Operator 2";
    AssetGroupTargetConditionOperator3 = "Asset Group Target Condition Operator 3";
    AssetGroupTargetValue1 = "Asset Group Target Value 1";
    AssetGroupTargetValue2 = "Asset Group Target Value 2";
    AssetGroupTargetValue3 = "Asset Group Target Value 3";

    # New Customer Acquisition Goal
    NewCustomerAcquisitionGoal = "New Customer Acquisition Goal";
    AdditionalConversionValue = "Additional Conversion Value";
    NewCustomerAcquisitionGoalId = "New Customer Acquisition Goal Id";
    NewCustomerAcquisitionBidOnlyMode = "New Customer Acquisition Bid Only Mode";



__author__ = 'Bing Ads SDK Team'
__email__ = 'bing_ads_sdk@microsoft.com'

from .bulk_ad_group_age_criterion import *
from .bulk_ad_group_biddable_criterion import *
from .bulk_ad_group_day_time_criterion import *
from .bulk_ad_group_device_criterion import *
from .bulk_ad_group_gender_criterion import *
from .bulk_ad_group_location_criterion import *
from .bulk_ad_group_location_intent_criterion import *
from .bulk_ad_group_negative_location_criterion import *
from .bulk_ad_group_radius_criterion import *
from .bulk_ad_group_profile_criterion import *
from .bulk_ad_group_negative_age_criterion import *
from .bulk_ad_group_negative_criterion import *
from .bulk_ad_group_negative_gender_criterion import *
from .bulk_ad_group_negative_profile_criterion import *
from .bulk_campaign_age_criterion import *
from .bulk_campaign_biddable_criterion import *
from .bulk_campaign_day_time_criterion import *
from .bulk_campaign_deal_criterion import *
from .bulk_campaign_device_criterion import *
from .bulk_campaign_gender_criterion import *
from .bulk_campaign_location_criterion import *
from .bulk_campaign_location_intent_criterion import *
from .bulk_campaign_negative_location_criterion import *
from .bulk_campaign_radius_criterion import *
from .bulk_campaign_profile_criterion import *
from .bulk_campaign_negative_criterion import *
from .bulk_ad_group_hotel_advance_booking_window_criterion import *
from .bulk_ad_group_hotel_check_in_date_criterion import *
from .bulk_ad_group_hotel_check_in_day_criterion import *
from .bulk_ad_group_hotel_date_selection_type_criterion import *
from .bulk_ad_group_hotel_length_of_stay_criterion import *
from .bulk_ad_group_genre_criterion import *



class _ErrorMessages:
    @staticmethod
    def get_format_version_not_supported_message(version):
        return "Format version {0} is not supported".format(version)

    @staticmethod
    def get_property_must_not_be_null_message(entity_type, property_name):
        return "Property {0}.{1} must not be null when calling WriteEntity.".format(str(entity_type),
                                                                                    str(property_name))

    @staticmethod
    def get_list_must_not_be_null_or_empty(entity_type, property_name):
        return "List {0}.{1} must not be null or empty when calling WriteEntity.".format(str(entity_type),
                                                                                         str(property_name))



from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.bulk_object import _BulkObject
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *


class BulkError(_BulkObject):
    """ Contains bulk file error details in a separate record that corresponds to the record of a :class:`.BulkEntity` derived instance.

    Properties of this class and of classes that it is derived from, correspond to error fields of the 'Error' records in a bulk file.
    For more information, see Bulk File Schema at https://go.microsoft.com/fwlink/?linkid=846127.

    *Example:*

    If you upload a :class:`.BulkCampaign` without setting the campaign name using :meth:`BulkServiceManager.upload_entities,
    and if you request errors to be returned in the results using the corresponding :attr:`SubmitUploadParameters.response_mode`,
    then the upload result file will contain a record that can be read with a :class:`.BulkFileReader` as an instance of :class:`.BulkError`.
    """

    def __init__(self):
        self._error = None
        self._number = None
        self._editorial_location = None
        self._editorial_term = None
        self._editorial_reason_code = None
        self._publisher_countries = None
        self._entity = None
        self._field_path = None

    @property
    def entity(self):
        return self._entity

    @entity.setter
    def entity(self, value):
        self._entity = value

    @property
    def error(self):
        """ The error code, for example 'CampaignServiceEditorialValidationError'.

        Corresponds to the 'Error' field in the bulk file.
        For more information, see Bing Ads Operation Error Codes at https://go.microsoft.com/fwlink/?linkid=846127.

        :rtype: str
        """

        return self._error

    @property
    def number(self):
        """ The error number, for example '1042'.

        Corresponds to the 'Error Number' field in the bulk file.
        For more information, see Bing Ads Operation Error Codes at https://go.microsoft.com/fwlink/?linkid=846127.

        :rtype: int
        """

        return self._number

    @property
    def editorial_location(self):
        """ The location of the entity property that resulted in the editorial error, for example 'AdDescription'.

        Corresponds to the 'Editorial Location' field in the bulk file.

        :rtype: str
        """

        return self._editorial_location

    @property
    def editorial_term(self):
        """ The term that resulted in the editorial error, for example 'bing'.

        Corresponds to the 'Editorial Term' field in the bulk file.

        :rtype: str
        """

        return self._editorial_term

    @property
    def editorial_reason_code(self):
        """ The term that resulted in the editorial error, for example '17'.

        Corresponds to the 'Editorial Reason Code' field in the bulk file.
        For more information, see Bing Ads Editorial Failure Reason Codes at https://go.microsoft.com/fwlink/?linkid=846127.

        :rtype: int
        """

        return self._editorial_reason_code


    @property
    def field_path(self):
        """ The term that resulted in the editorial error.

        Corresponds to the 'Field Path' field in the bulk file.

        :rtype: int
        """

        return self._field_path

    @property
    def publisher_countries(self):
        """ The publisher countries where editorial restriction is enforced, for example 'US'.

        Corresponds to the 'Publisher Countries' field in the bulk file.

        *Remarks:*

        In a bulk file, the list of publisher countries are delimited with a semicolon (;).

        :rtype: str
        """

        return self._publisher_countries

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Error,
            field_to_csv=lambda c: bulk_str(c.error),
            csv_to_field=lambda c, v: setattr(c, '_error', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ErrorNumber,
            field_to_csv=lambda c: bulk_str(c.number),
            csv_to_field=lambda c, v: setattr(c, '_number', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.EditorialLocation,
            field_to_csv=lambda c: bulk_str(c.editorial_location),
            csv_to_field=lambda c, v: setattr(c, '_editorial_location', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.EditorialReasonCode,
            field_to_csv=lambda c: bulk_str(c.editorial_reason_code),
            csv_to_field=lambda c, v: setattr(c, '_editorial_reason_code', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.EditorialTerm,
            field_to_csv=lambda c: bulk_str(c.editorial_term),
            csv_to_field=lambda c, v: setattr(c, '_editorial_term', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.PublisherCountries,
            field_to_csv=lambda c: bulk_str(c.publisher_countries),
            csv_to_field=lambda c, v: setattr(c, '_publisher_countries', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FieldPath,
            field_to_csv=lambda c: bulk_str(c.field_path),
            csv_to_field=lambda c, v: setattr(c, '_field_path', v if v else None)
        ),
    ]

    def can_enclose_in_multiline_entity(self):
        return super(BulkError, self).can_enclose_in_multiline_entity()

    def enclose_in_multiline_entity(self):
        return super(BulkError, self).enclose_in_multiline_entity()

    def read_from_row_values(self, row_values):
        row_values.convert_to_entity(self, BulkError._MAPPINGS)

    def read_related_data_from_stream(self, stream_reader):
        return super(BulkError, self).read_related_data_from_stream(stream_reader)

    def write_to_row_values(self, row_values, exclude_readonly_data):
        self.entity.write_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkError._MAPPINGS)

    def write_to_stream(self, stream_writer, exclude_readonly_data):
        return super(BulkError, self).write_to_stream(stream_writer, exclude_readonly_data)



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *


class BulkAdCustomizerAttribute(_SingleRecordBulkEntity):
    """ Represents an AdCustomizerAttribute.

    Properties of this class and of classes that it is derived from, correspond to fields of the AdCustomizerAttribute record in a bulk file.
    For more information, see AdCustomizerAttribute at https://go.microsoft.com/fwlink/?linkid=846127
    
    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, id = None, name=None, account_value=None, data_type=None, editorial_status = None, status=None):
        super(BulkAdCustomizerAttribute, self).__init__()
        self._id = id
        self._name = name
        self._account_value = account_value
        self._data_type = data_type
        self._editorial_status = editorial_status
        self._status = status


    @property
    def id(self):
        """ the id of bulk record
        Corresponds to the 'Id' field in the bulk file.

        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, value):
        self._id = value        

    @property
    def name(self):
        """ the name of the ad customizer attribute
        Corresponds to the 'Name' field in the bulk file.

        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, value):
        self._name = value
        
    @property
    def account_value(self):
        """ the value of the account of ad customizer attribute
        Corresponds to the 'AdCustomizer AttributeValue' field in the bulk file.

        :rtype: str
        """
        return self._account_value

    @account_value.setter
    def account_value(self, value):
        self._account_value = value
          
    @property
    def data_type(self):
        """ the data type of ad customizer attribute
        Corresponds to the 'AdCustomizer DataType' field in the bulk file.

        :rtype: str
        """
        return self._data_type

    @data_type.setter
    def data_type(self, value):
        self._data_type = value
 
    @property
    def editorial_status(self):
        """ the editorial status of ad customizer attribute
        Corresponds to the 'Editorial Status' field in the bulk file.

        :rtype: str
        """
        return self._editorial_status

    @editorial_status.setter
    def editorial_status(self, value):
        self._editorial_status = value

    @property
    def status(self):
        """ the status of ad customizer attribute
        Corresponds to the 'Status' field in the bulk file.

        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, value):
        self._status = value

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.id),
            csv_to_field=lambda c, v: setattr(c, 'id', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Name,
            field_to_csv=lambda c: bulk_str(c.name),
            csv_to_field=lambda c, v: setattr(c, 'name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AdCustomizerAttributeValue,
            field_to_csv=lambda c: bulk_str(c.account_value),
            csv_to_field=lambda c, v: setattr(c, 'account_value', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AdCustomizerDataType,
            field_to_csv=lambda c: bulk_str(c.data_type),
            csv_to_field=lambda c, v: setattr(c, 'data_type', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.EditorialStatus,
            field_to_csv=lambda c: bulk_str(c.editorial_status),
            csv_to_field=lambda c, v: setattr(c, 'editorial_status', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: c.status,
            csv_to_field=lambda c, v: setattr(c, 'status', v)
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        row_values.convert_to_entity(self, BulkAdCustomizerAttribute._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self.convert_to_values(row_values, BulkAdCustomizerAttribute._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkAdCustomizerAttribute, self).read_additional_data(stream_reader)



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *


class BulkFeed(_SingleRecordBulkEntity):
    """ Represents a feed.

    Properties of this class and of classes that it is derived from, correspond to fields of the Feed record in a bulk file.
    For more information, see Feed at https://go.microsoft.com/fwlink/?linkid=846127
    
    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, id = None, status=None, account_id=None, sub_type=None, feed_name = None, custom_attr=None, schedule=None):
        super(BulkFeed, self).__init__()
        self._status = status
        self._account_id = account_id
        self._id = id
        self._sub_type = sub_type
        self._name = feed_name
        self._custom_attributes = custom_attr
        self._schedule = schedule


    @property
    def id(self):
        """ the status of bulk record
        Corresponds to the 'Id' field in the bulk file.

        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, value):
        self._id = value        

    @property
    def status(self):
        """ the status of bulk record
        Corresponds to the 'Status' field in the bulk file.

        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, value):
        self._status = value

    @property
    def account_id(self):
        """ the id of the account which contains the feed
        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: long
        """
        return self._account_id

    @account_id.setter
    def account_id(self, value):
        self._account_id = value
          
    @property
    def name(self):
        """ the id of the account which contains the feed
        Corresponds to the 'Feed Name' field in the bulk file.

        :rtype: long
        """
        return self._name

    @name.setter
    def name(self, value):
        self._name = value
        
    @property
    def sub_type(self):
        """ the id of the account which contains the feed
        Corresponds to the 'Sub Type' field in the bulk file.

        :rtype: long
        """
        return self._sub_type

    @sub_type.setter
    def sub_type(self, value):
        self._sub_type = value
 
    @property
    def custom_attributes(self):
        """ the id of the account which contains the feed
        Corresponds to the 'Custom Attributes' field in the bulk file.

        :rtype: long
        """
        return self._custom_attributes
    
    @custom_attributes.setter
    def custom_attributes(self, value):
        self._custom_attributes = value

    @property
    def schedule(self):
        """ the schedule of the feed
        Corresponds to the 'Schedule' field in the bulk file.

        :rtype: string
        """
        return self._schedule

    @schedule.setter
    def schedule(self, value):
        self._schedule = value
 
    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.id),
            csv_to_field=lambda c, v: setattr(c, 'id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.account_id),
            csv_to_field=lambda c, v: setattr(c, 'account_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: c.status,
            csv_to_field=lambda c, v: setattr(c, 'status', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FeedName,
            field_to_csv=lambda c: bulk_str(c.name),
            csv_to_field=lambda c, v: setattr(c, 'name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.SubType,
            field_to_csv=lambda c: bulk_str(c.sub_type),
            csv_to_field=lambda c, v: setattr(c, 'sub_type', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CustomAttributes,
            field_to_csv=lambda c: field_to_csv_CustomAttributes(c.custom_attributes),
            csv_to_field=lambda c, v: csv_to_field_CustomAttributes(c, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Schedule,
            field_to_csv=lambda c: c.schedule,
            csv_to_field=lambda c, v: setattr(c, 'schedule', v)
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        row_values.convert_to_entity(self, BulkFeed._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self.convert_to_values(row_values, BulkFeed._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkFeed, self).read_additional_data(stream_reader)



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.ad_extension_status import AdExtensionStatus
from openapi_client.models.campaign.confirmation_action import ConfirmationAction
from openapi_client.models.campaign.image_asset import ImageAsset
from openapi_client.models.campaign.key_value_pair_ofstring_andstring import KeyValuePairOfstringAndstring
from openapi_client.models.campaign.lead_delivery import LeadDelivery
from openapi_client.models.campaign.lead_form_call_to_action import LeadFormCallToAction
from openapi_client.models.campaign.lead_form_question_definition import LeadFormQuestionDefinition
from openapi_client.models.campaign.schedule import Schedule
from typing_extensions import Self

class LeadFormAdExtension(BaseModel):
    """
    LeadFormAdExtension
    """ # noqa: E501
    call_to_action: Optional[LeadFormCallToAction] = Field(default=None, alias="CallToAction")
    action_name: Optional[StrictStr] = Field(default=None, alias="ActionName")
    action_description: Optional[StrictStr] = Field(default=None, alias="ActionDescription")
    corporate_image: Optional[ImageAsset] = Field(default=None, alias="CorporateImage")
    corporate_image_internal_url: Optional[StrictStr] = Field(default=None, alias="CorporateImageInternalUrl")
    form_headline: Optional[StrictStr] = Field(default=None, alias="FormHeadline")
    form_business_name: Optional[StrictStr] = Field(default=None, alias="FormBusinessName")
    form_description: Optional[StrictStr] = Field(default=None, alias="FormDescription")
    form_policy_url: Optional[StrictStr] = Field(default=None, alias="FormPolicyUrl")
    form_questions: Optional[List[Optional[LeadFormQuestionDefinition]]] = Field(default=None, alias="FormQuestions")
    confirmation_message: Optional[StrictStr] = Field(default=None, alias="ConfirmationMessage")
    confirmation_description: Optional[StrictStr] = Field(default=None, alias="ConfirmationDescription")
    confirmation_action: Optional[ConfirmationAction] = Field(default=None, alias="ConfirmationAction")
    confirmation_url: Optional[StrictStr] = Field(default=None, alias="ConfirmationUrl")
    lead_delivery: Optional[LeadDelivery] = Field(default=None, alias="LeadDelivery")
    lead_emails: Optional[StrictStr] = Field(default=None, alias="LeadEmails")
    lead_webhook_url: Optional[StrictStr] = Field(default=None, alias="LeadWebhookUrl")
    lead_webhook_key: Optional[StrictStr] = Field(default=None, alias="LeadWebhookKey")
    language: Optional[StrictStr] = Field(default=None, alias="Language")
    status: Optional[AdExtensionStatus] = Field(default=None, alias="Status")
    scheduling: Optional[Schedule] = Field(default=None, alias="Scheduling")
    device_preference: Optional[StrictStr] = Field(default=None, alias="DevicePreference")
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    type: Optional[StrictStr] = Field(default='LeadFormAdExtension', alias="Type")
    version: Optional[StrictInt] = Field(default=None, alias="Version")
    forward_compatibility_map: Optional[List[Optional[KeyValuePairOfstringAndstring]]] = Field(default=None, alias="ForwardCompatibilityMap")
    __properties: ClassVar[List[str]] = ["CallToAction", "ActionName", "ActionDescription", "CorporateImage", "CorporateImageInternalUrl", "FormHeadline", "FormBusinessName", "FormDescription", "FormPolicyUrl", "FormQuestions", "ConfirmationMessage", "ConfirmationDescription", "ConfirmationAction", "ConfirmationUrl", "LeadDelivery", "LeadEmails", "LeadWebhookUrl", "LeadWebhookKey", "Language", "Status", "Scheduling", "DevicePreference", "Id", "Type", "Version", "ForwardCompatibilityMap"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	
    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())
	
    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of LeadFormAdExtension from a JSON string"""
        return cls.from_dict(json.loads(json_str))
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of corporate_image
        if self.corporate_image:
            _dict['CorporateImage'] = self.corporate_image.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in form_questions (list)
        _items = []
        if self.form_questions:
            for _item_form_questions in self.form_questions:
                if _item_form_questions:
                    _items.append(_item_form_questions.to_dict())
            _dict['FormQuestions'] = _items
        # override the default output from pydantic by calling `to_dict()` of scheduling
        if self.scheduling:
            _dict['Scheduling'] = self.scheduling.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in forward_compatibility_map (list)
        _items = []
        if self.forward_compatibility_map:
            for _item_forward_compatibility_map in self.forward_compatibility_map:
                if _item_forward_compatibility_map:
                    _items.append(_item_forward_compatibility_map.to_dict())
            _dict['ForwardCompatibilityMap'] = _items
        # set to None if call_to_action (nullable) is None
        # and model_fields_set contains the field
        if self.call_to_action is None and "call_to_action" in self.model_fields_set:
            _dict['CallToAction'] = None

        # set to None if action_name (nullable) is None
        # and model_fields_set contains the field
        if self.action_name is None and "action_name" in self.model_fields_set:
            _dict['ActionName'] = None

        # set to None if action_description (nullable) is None
        # and model_fields_set contains the field
        if self.action_description is None and "action_description" in self.model_fields_set:
            _dict['ActionDescription'] = None

        # set to None if corporate_image (nullable) is None
        # and model_fields_set contains the field
        if self.corporate_image is None and "corporate_image" in self.model_fields_set:
            _dict['CorporateImage'] = None

        # set to None if corporate_image_internal_url (nullable) is None
        # and model_fields_set contains the field
        if self.corporate_image_internal_url is None and "corporate_image_internal_url" in self.model_fields_set:
            _dict['CorporateImageInternalUrl'] = None

        # set to None if form_headline (nullable) is None
        # and model_fields_set contains the field
        if self.form_headline is None and "form_headline" in self.model_fields_set:
            _dict['FormHeadline'] = None

        # set to None if form_business_name (nullable) is None
        # and model_fields_set contains the field
        if self.form_business_name is None and "form_business_name" in self.model_fields_set:
            _dict['FormBusinessName'] = None

        # set to None if form_description (nullable) is None
        # and model_fields_set contains the field
        if self.form_description is None and "form_description" in self.model_fields_set:
            _dict['FormDescription'] = None

        # set to None if form_policy_url (nullable) is None
        # and model_fields_set contains the field
        if self.form_policy_url is None and "form_policy_url" in self.model_fields_set:
            _dict['FormPolicyUrl'] = None

        # set to None if form_questions (nullable) is None
        # and model_fields_set contains the field
        if self.form_questions is None and "form_questions" in self.model_fields_set:
            _dict['FormQuestions'] = None

        # set to None if confirmation_message (nullable) is None
        # and model_fields_set contains the field
        if self.confirmation_message is None and "confirmation_message" in self.model_fields_set:
            _dict['ConfirmationMessage'] = None

        # set to None if confirmation_description (nullable) is None
        # and model_fields_set contains the field
        if self.confirmation_description is None and "confirmation_description" in self.model_fields_set:
            _dict['ConfirmationDescription'] = None

        # set to None if confirmation_action (nullable) is None
        # and model_fields_set contains the field
        if self.confirmation_action is None and "confirmation_action" in self.model_fields_set:
            _dict['ConfirmationAction'] = None

        # set to None if confirmation_url (nullable) is None
        # and model_fields_set contains the field
        if self.confirmation_url is None and "confirmation_url" in self.model_fields_set:
            _dict['ConfirmationUrl'] = None

        # set to None if lead_delivery (nullable) is None
        # and model_fields_set contains the field
        if self.lead_delivery is None and "lead_delivery" in self.model_fields_set:
            _dict['LeadDelivery'] = None

        # set to None if lead_emails (nullable) is None
        # and model_fields_set contains the field
        if self.lead_emails is None and "lead_emails" in self.model_fields_set:
            _dict['LeadEmails'] = None

        # set to None if lead_webhook_url (nullable) is None
        # and model_fields_set contains the field
        if self.lead_webhook_url is None and "lead_webhook_url" in self.model_fields_set:
            _dict['LeadWebhookUrl'] = None

        # set to None if lead_webhook_key (nullable) is None
        # and model_fields_set contains the field
        if self.lead_webhook_key is None and "lead_webhook_key" in self.model_fields_set:
            _dict['LeadWebhookKey'] = None

        # set to None if language (nullable) is None
        # and model_fields_set contains the field
        if self.language is None and "language" in self.model_fields_set:
            _dict['Language'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['Status'] = None

        # set to None if scheduling (nullable) is None
        # and model_fields_set contains the field
        if self.scheduling is None and "scheduling" in self.model_fields_set:
            _dict['Scheduling'] = None

        # set to None if device_preference (nullable) is None
        # and model_fields_set contains the field
        if self.device_preference is None and "device_preference" in self.model_fields_set:
            _dict['DevicePreference'] = None

        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if type (nullable) is None
        # and model_fields_set contains the field
        if self.type is None and "type" in self.model_fields_set:
            _dict['Type'] = None

        # set to None if version (nullable) is None
        # and model_fields_set contains the field
        if self.version is None and "version" in self.model_fields_set:
            _dict['Version'] = None

        # set to None if forward_compatibility_map (nullable) is None
        # and model_fields_set contains the field
        if self.forward_compatibility_map is None and "forward_compatibility_map" in self.model_fields_set:
            _dict['ForwardCompatibilityMap'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of LeadFormAdExtension from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "CallToAction": obj.get("CallToAction") if obj.get("CallToAction") is not None else None,
                        "ActionName": obj.get("ActionName") if obj.get("ActionName") is not None else None,
                        "ActionDescription": obj.get("ActionDescription") if obj.get("ActionDescription") is not None else None,
                        "CorporateImage": ImageAsset.from_dict(obj["CorporateImage"]) if obj.get("CorporateImage") is not None else None,
                        "CorporateImageInternalUrl": obj.get("CorporateImageInternalUrl") if obj.get("CorporateImageInternalUrl") is not None else None,
                        "FormHeadline": obj.get("FormHeadline") if obj.get("FormHeadline") is not None else None,
                        "FormBusinessName": obj.get("FormBusinessName") if obj.get("FormBusinessName") is not None else None,
                        "FormDescription": obj.get("FormDescription") if obj.get("FormDescription") is not None else None,
                        "FormPolicyUrl": obj.get("FormPolicyUrl") if obj.get("FormPolicyUrl") is not None else None,
                        "FormQuestions": [LeadFormQuestionDefinition.from_dict(_item) for _item in obj["FormQuestions"]] if obj.get("FormQuestions") is not None else None,
                        "ConfirmationMessage": obj.get("ConfirmationMessage") if obj.get("ConfirmationMessage") is not None else None,
                        "ConfirmationDescription": obj.get("ConfirmationDescription") if obj.get("ConfirmationDescription") is not None else None,
                        "ConfirmationAction": obj.get("ConfirmationAction") if obj.get("ConfirmationAction") is not None else None,
                        "ConfirmationUrl": obj.get("ConfirmationUrl") if obj.get("ConfirmationUrl") is not None else None,
                        "LeadDelivery": obj.get("LeadDelivery") if obj.get("LeadDelivery") is not None else None,
                        "LeadEmails": obj.get("LeadEmails") if obj.get("LeadEmails") is not None else None,
                        "LeadWebhookUrl": obj.get("LeadWebhookUrl") if obj.get("LeadWebhookUrl") is not None else None,
                        "LeadWebhookKey": obj.get("LeadWebhookKey") if obj.get("LeadWebhookKey") is not None else None,
                        "Language": obj.get("Language") if obj.get("Language") is not None else None,
                        "Status": obj.get("Status") if obj.get("Status") is not None else None,
                        "Scheduling": Schedule.from_dict(obj["Scheduling"]) if obj.get("Scheduling") is not None else None,
                        "DevicePreference": obj.get("DevicePreference") if obj.get("DevicePreference") is not None else None,
                        "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "Type": obj.get("Type") if obj.get("Type") is not None else 'LeadFormAdExtension',
                        "Version": obj.get("Version") if obj.get("Version") is not None else None,
                        "ForwardCompatibilityMap": [KeyValuePairOfstringAndstring.from_dict(_item) for _item in obj["ForwardCompatibilityMap"]] if obj.get("ForwardCompatibilityMap") is not None else None
        })
        return _obj



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from typing_extensions import Self

class Video(BaseModel):
    """
    Video
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    description: Optional[StrictStr] = Field(default=None, alias="Description")
    thumbnail_url: Optional[StrictStr] = Field(default=None, alias="ThumbnailUrl")
    source_url: Optional[StrictStr] = Field(default=None, alias="SourceUrl")
    url: Optional[StrictStr] = Field(default=None, alias="Url")
    aspect_ratio: Optional[StrictStr] = Field(default=None, alias="AspectRatio")
    duration_in_milliseconds: Optional[StrictInt] = Field(default=None, alias="DurationInMilliseconds")
    status: Optional[StrictStr] = Field(default=None, alias="Status")
    created_date_time_in_utc: Optional[datetime] = Field(default=None, alias="CreatedDateTimeInUTC")
    modified_date_time_in_utc: Optional[datetime] = Field(default=None, alias="ModifiedDateTimeInUTC")
    failure_code: Optional[StrictStr] = Field(default=None, alias="FailureCode")
    __properties: ClassVar[List[str]] = ["Id", "Description", "ThumbnailUrl", "SourceUrl", "Url", "AspectRatio", "DurationInMilliseconds", "Status", "CreatedDateTimeInUTC", "ModifiedDateTimeInUTC", "FailureCode"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict['Description'] = None

        # set to None if thumbnail_url (nullable) is None
        # and model_fields_set contains the field
        if self.thumbnail_url is None and "thumbnail_url" in self.model_fields_set:
            _dict['ThumbnailUrl'] = None

        # set to None if source_url (nullable) is None
        # and model_fields_set contains the field
        if self.source_url is None and "source_url" in self.model_fields_set:
            _dict['SourceUrl'] = None

        # set to None if url (nullable) is None
        # and model_fields_set contains the field
        if self.url is None and "url" in self.model_fields_set:
            _dict['Url'] = None

        # set to None if aspect_ratio (nullable) is None
        # and model_fields_set contains the field
        if self.aspect_ratio is None and "aspect_ratio" in self.model_fields_set:
            _dict['AspectRatio'] = None

        # set to None if duration_in_milliseconds (nullable) is None
        # and model_fields_set contains the field
        if self.duration_in_milliseconds is None and "duration_in_milliseconds" in self.model_fields_set:
            _dict['DurationInMilliseconds'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['Status'] = None

        # set to None if created_date_time_in_utc (nullable) is None
        # and model_fields_set contains the field
        if self.created_date_time_in_utc is None and "created_date_time_in_utc" in self.model_fields_set:
            _dict['CreatedDateTimeInUTC'] = None

        # set to None if modified_date_time_in_utc (nullable) is None
        # and model_fields_set contains the field
        if self.modified_date_time_in_utc is None and "modified_date_time_in_utc" in self.model_fields_set:
            _dict['ModifiedDateTimeInUTC'] = None

        # set to None if failure_code (nullable) is None
        # and model_fields_set contains the field
        if self.failure_code is None and "failure_code" in self.model_fields_set:
            _dict['FailureCode'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Video from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "Description": obj.get("Description") if obj.get("Description") is not None else None,
                        "ThumbnailUrl": obj.get("ThumbnailUrl") if obj.get("ThumbnailUrl") is not None else None,
                        "SourceUrl": obj.get("SourceUrl") if obj.get("SourceUrl") is not None else None,
                        "Url": obj.get("Url") if obj.get("Url") is not None else None,
                        "AspectRatio": obj.get("AspectRatio") if obj.get("AspectRatio") is not None else None,
                        "DurationInMilliseconds": obj.get("DurationInMilliseconds") if obj.get("DurationInMilliseconds") is not None else None,
                        "Status": obj.get("Status") if obj.get("Status") is not None else None,
                        "CreatedDateTimeInUTC": obj.get("CreatedDateTimeInUTC") if obj.get("CreatedDateTimeInUTC") is not None else None,
                        "ModifiedDateTimeInUTC": obj.get("ModifiedDateTimeInUTC") if obj.get("ModifiedDateTimeInUTC") is not None else None,
                        "FailureCode": obj.get("FailureCode") if obj.get("FailureCode") is not None else None
        })
        return _obj



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from openapi_client.models.campaign.age_criterion import AgeCriterion
from openapi_client.models.campaign.age_range import AgeRange
from openapi_client.models.campaign.audience_criterion import AudienceCriterion
from openapi_client.models.campaign.audience_type import AudienceType
from openapi_client.models.campaign.day import Day
from openapi_client.models.campaign.day_time_criterion import DayTimeCriterion
from openapi_client.models.campaign.deal_criterion import DealCriterion
from openapi_client.models.campaign.device_criterion import DeviceCriterion
from openapi_client.models.campaign.distance_unit import DistanceUnit
from openapi_client.models.campaign.gender_criterion import GenderCriterion
from openapi_client.models.campaign.gender_type import GenderType
from openapi_client.models.campaign.genre_criterion import GenreCriterion
from openapi_client.models.campaign.hotel_advance_booking_window_criterion import HotelAdvanceBookingWindowCriterion
from openapi_client.models.campaign.hotel_check_in_date_criterion import HotelCheckInDateCriterion
from openapi_client.models.campaign.hotel_check_in_day_criterion import HotelCheckInDayCriterion
from openapi_client.models.campaign.hotel_date_selection_type import HotelDateSelectionType
from openapi_client.models.campaign.hotel_date_selection_type_criterion import HotelDateSelectionTypeCriterion
from openapi_client.models.campaign.hotel_group import HotelGroup
from openapi_client.models.campaign.hotel_length_of_stay_criterion import HotelLengthOfStayCriterion
from openapi_client.models.campaign.hotel_listing import HotelListing
from openapi_client.models.campaign.hotel_listing_type import HotelListingType
from openapi_client.models.campaign.intent_option import IntentOption
from openapi_client.models.campaign.location_criterion import LocationCriterion
from openapi_client.models.campaign.location_intent_criterion import LocationIntentCriterion
from openapi_client.models.campaign.minute import Minute
from openapi_client.models.campaign.placement_criterion import PlacementCriterion
from openapi_client.models.campaign.product_condition import ProductCondition
from openapi_client.models.campaign.product_partition import ProductPartition
from openapi_client.models.campaign.product_partition_type import ProductPartitionType
from openapi_client.models.campaign.product_scope import ProductScope
from openapi_client.models.campaign.profile_criterion import ProfileCriterion
from openapi_client.models.campaign.profile_type import ProfileType
from openapi_client.models.campaign.radius_criterion import RadiusCriterion
from openapi_client.models.campaign.store_criterion import StoreCriterion
from openapi_client.models.campaign.webpage import Webpage
from openapi_client.models.campaign.webpage_parameter import WebpageParameter
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

Criterion_ONE_OF_SCHEMAS = ["AgeCriterion", "AudienceCriterion", "DayTimeCriterion", "DealCriterion", "DeviceCriterion", "GenderCriterion", "GenreCriterion", "HotelAdvanceBookingWindowCriterion", "HotelCheckInDateCriterion", "HotelCheckInDayCriterion", "HotelDateSelectionTypeCriterion", "HotelGroup", "HotelLengthOfStayCriterion", "LocationCriterion", "LocationIntentCriterion", "PlacementCriterion", "ProductPartition", "ProductScope", "ProfileCriterion", "RadiusCriterion", "StoreCriterion", "Webpage"]

class Criterion(BaseModel):
    """
    Criterion
    """
    # data type: AgeCriterion
    oneof_schema_age_criterion_validator: Optional[AgeCriterion] = None
    # data type: AudienceCriterion
    oneof_schema_audience_criterion_validator: Optional[AudienceCriterion] = None
    # data type: DayTimeCriterion
    oneof_schema_day_time_criterion_validator: Optional[DayTimeCriterion] = None
    # data type: DealCriterion
    oneof_schema_deal_criterion_validator: Optional[DealCriterion] = None
    # data type: DeviceCriterion
    oneof_schema_device_criterion_validator: Optional[DeviceCriterion] = None
    # data type: GenderCriterion
    oneof_schema_gender_criterion_validator: Optional[GenderCriterion] = None
    # data type: GenreCriterion
    oneof_schema_genre_criterion_validator: Optional[GenreCriterion] = None
    # data type: HotelAdvanceBookingWindowCriterion
    oneof_schema_hotel_advance_booking_window_criterion_validator: Optional[HotelAdvanceBookingWindowCriterion] = None
    # data type: HotelCheckInDateCriterion
    oneof_schema_hotel_check_in_date_criterion_validator: Optional[HotelCheckInDateCriterion] = None
    # data type: HotelCheckInDayCriterion
    oneof_schema_hotel_check_in_day_criterion_validator: Optional[HotelCheckInDayCriterion] = None
    # data type: HotelDateSelectionTypeCriterion
    oneof_schema_hotel_date_selection_type_criterion_validator: Optional[HotelDateSelectionTypeCriterion] = None
    # data type: HotelGroup
    oneof_schema_hotel_group_validator: Optional[HotelGroup] = None
    # data type: HotelLengthOfStayCriterion
    oneof_schema_hotel_length_of_stay_criterion_validator: Optional[HotelLengthOfStayCriterion] = None
    # data type: LocationCriterion
    oneof_schema_location_criterion_validator: Optional[LocationCriterion] = None
    # data type: LocationIntentCriterion
    oneof_schema_location_intent_criterion_validator: Optional[LocationIntentCriterion] = None
    # data type: PlacementCriterion
    oneof_schema_placement_criterion_validator: Optional[PlacementCriterion] = None
    # data type: ProductPartition
    oneof_schema_product_partition_validator: Optional[ProductPartition] = None
    # data type: ProductScope
    oneof_schema_product_scope_validator: Optional[ProductScope] = None
    # data type: ProfileCriterion
    oneof_schema_profile_criterion_validator: Optional[ProfileCriterion] = None
    # data type: RadiusCriterion
    oneof_schema_radius_criterion_validator: Optional[RadiusCriterion] = None
    # data type: StoreCriterion
    oneof_schema_store_criterion_validator: Optional[StoreCriterion] = None
    # data type: Webpage
    oneof_schema_webpage_validator: Optional[Webpage] = None
    actual_instance: Optional[Union[AgeCriterion, AudienceCriterion, DayTimeCriterion, DealCriterion, DeviceCriterion, GenderCriterion, GenreCriterion, HotelAdvanceBookingWindowCriterion, HotelCheckInDateCriterion, HotelCheckInDayCriterion, HotelDateSelectionTypeCriterion, HotelGroup, HotelLengthOfStayCriterion, LocationCriterion, LocationIntentCriterion, PlacementCriterion, ProductPartition, ProductScope, ProfileCriterion, RadiusCriterion, StoreCriterion, Webpage]] = None
    one_of_schemas: Set[str] = { "AgeCriterion", "AudienceCriterion", "DayTimeCriterion", "DealCriterion", "DeviceCriterion", "GenderCriterion", "GenreCriterion", "HotelAdvanceBookingWindowCriterion", "HotelCheckInDateCriterion", "HotelCheckInDayCriterion", "HotelDateSelectionTypeCriterion", "HotelGroup", "HotelLengthOfStayCriterion", "LocationCriterion", "LocationIntentCriterion", "PlacementCriterion", "ProductPartition", "ProductScope", "ProfileCriterion", "RadiusCriterion", "StoreCriterion", "Webpage" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )

    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        if v is None:
            return v

        instance = Criterion.model_construct()
        error_messages = []
        match = 0
        # validate data type: AgeCriterion
        if not isinstance(v, AgeCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `AgeCriterion`")
        else:
            match += 1
        # validate data type: AudienceCriterion
        if not isinstance(v, AudienceCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `AudienceCriterion`")
        else:
            match += 1
        # validate data type: DayTimeCriterion
        if not isinstance(v, DayTimeCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `DayTimeCriterion`")
        else:
            match += 1
        # validate data type: DealCriterion
        if not isinstance(v, DealCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `DealCriterion`")
        else:
            match += 1
        # validate data type: DeviceCriterion
        if not isinstance(v, DeviceCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `DeviceCriterion`")
        else:
            match += 1
        # validate data type: GenderCriterion
        if not isinstance(v, GenderCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `GenderCriterion`")
        else:
            match += 1
        # validate data type: GenreCriterion
        if not isinstance(v, GenreCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `GenreCriterion`")
        else:
            match += 1
        # validate data type: HotelAdvanceBookingWindowCriterion
        if not isinstance(v, HotelAdvanceBookingWindowCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `HotelAdvanceBookingWindowCriterion`")
        else:
            match += 1
        # validate data type: HotelCheckInDateCriterion
        if not isinstance(v, HotelCheckInDateCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `HotelCheckInDateCriterion`")
        else:
            match += 1
        # validate data type: HotelCheckInDayCriterion
        if not isinstance(v, HotelCheckInDayCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `HotelCheckInDayCriterion`")
        else:
            match += 1
        # validate data type: HotelDateSelectionTypeCriterion
        if not isinstance(v, HotelDateSelectionTypeCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `HotelDateSelectionTypeCriterion`")
        else:
            match += 1
        # validate data type: HotelGroup
        if not isinstance(v, HotelGroup):
            error_messages.append(f"Error! Input type `{type(v)}` is not `HotelGroup`")
        else:
            match += 1
        # validate data type: HotelLengthOfStayCriterion
        if not isinstance(v, HotelLengthOfStayCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `HotelLengthOfStayCriterion`")
        else:
            match += 1
        # validate data type: LocationCriterion
        if not isinstance(v, LocationCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `LocationCriterion`")
        else:
            match += 1
        # validate data type: LocationIntentCriterion
        if not isinstance(v, LocationIntentCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `LocationIntentCriterion`")
        else:
            match += 1
        # validate data type: PlacementCriterion
        if not isinstance(v, PlacementCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `PlacementCriterion`")
        else:
            match += 1
        # validate data type: ProductPartition
        if not isinstance(v, ProductPartition):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ProductPartition`")
        else:
            match += 1
        # validate data type: ProductScope
        if not isinstance(v, ProductScope):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ProductScope`")
        else:
            match += 1
        # validate data type: ProfileCriterion
        if not isinstance(v, ProfileCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ProfileCriterion`")
        else:
            match += 1
        # validate data type: RadiusCriterion
        if not isinstance(v, RadiusCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `RadiusCriterion`")
        else:
            match += 1
        # validate data type: StoreCriterion
        if not isinstance(v, StoreCriterion):
            error_messages.append(f"Error! Input type `{type(v)}` is not `StoreCriterion`")
        else:
            match += 1
        # validate data type: Webpage
        if not isinstance(v, Webpage):
            error_messages.append(f"Error! Input type `{type(v)}` is not `Webpage`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in Criterion with oneOf schemas: AgeCriterion, AudienceCriterion, DayTimeCriterion, DealCriterion, DeviceCriterion, GenderCriterion, GenreCriterion, HotelAdvanceBookingWindowCriterion, HotelCheckInDateCriterion, HotelCheckInDayCriterion, HotelDateSelectionTypeCriterion, HotelGroup, HotelLengthOfStayCriterion, LocationCriterion, LocationIntentCriterion, PlacementCriterion, ProductPartition, ProductScope, ProfileCriterion, RadiusCriterion, StoreCriterion, Webpage. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in Criterion with oneOf schemas: AgeCriterion, AudienceCriterion, DayTimeCriterion, DealCriterion, DeviceCriterion, GenderCriterion, GenreCriterion, HotelAdvanceBookingWindowCriterion, HotelCheckInDateCriterion, HotelCheckInDayCriterion, HotelDateSelectionTypeCriterion, HotelGroup, HotelLengthOfStayCriterion, LocationCriterion, LocationIntentCriterion, PlacementCriterion, ProductPartition, ProductScope, ProfileCriterion, RadiusCriterion, StoreCriterion, Webpage. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: Optional[str]) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        if json_str is None:
            return instance

        error_messages = []
        match = 0

        # use oneOf discriminator to lookup the data type
        _data_type = json.loads(json_str).get("Type")
        if not _data_type:
            raise ValueError("Failed to lookup data type from the field `Type` in the input.")

		# check if data type is `AgeCriterion`
        if _data_type == "AgeCriterion":
            instance.actual_instance = AgeCriterion.from_json(json_str)
            return instance
			
		# check if data type is `AudienceCriterion`
        if _data_type == "AudienceCriterion":
            instance.actual_instance = AudienceCriterion.from_json(json_str)
            return instance
			
		# check if data type is `DayTimeCriterion`
        if _data_type == "DayTimeCriterion":
            instance.actual_instance = DayTimeCriterion.from_json(json_str)
            return instance
			
		# check if data type is `DealCriterion`
        if _data_type == "DealCriterion":
            instance.actual_instance = DealCriterion.from_json(json_str)
            return instance
			
		# check if data type is `DeviceCriterion`
        if _data_type == "DeviceCriterion":
            instance.actual_instance = DeviceCriterion.from_json(json_str)
            return instance
			
		# check if data type is `GenderCriterion`
        if _data_type == "GenderCriterion":
            instance.actual_instance = GenderCriterion.from_json(json_str)
            return instance
			
		# check if data type is `GenreCriterion`
        if _data_type == "GenreCriterion":
            instance.actual_instance = GenreCriterion.from_json(json_str)
            return instance
			
		# check if data type is `HotelAdvanceBookingWindowCriterion`
        if _data_type == "HotelAdvanceBookingWindowCriterion":
            instance.actual_instance = HotelAdvanceBookingWindowCriterion.from_json(json_str)
            return instance
			
		# check if data type is `HotelCheckInDateCriterion`
        if _data_type == "HotelCheckInDateCriterion":
            instance.actual_instance = HotelCheckInDateCriterion.from_json(json_str)
            return instance
			
		# check if data type is `HotelCheckInDayCriterion`
        if _data_type == "HotelCheckInDayCriterion":
            instance.actual_instance = HotelCheckInDayCriterion.from_json(json_str)
            return instance
			
		# check if data type is `HotelDateSelectionTypeCriterion`
        if _data_type == "HotelDateSelectionTypeCriterion":
            instance.actual_instance = HotelDateSelectionTypeCriterion.from_json(json_str)
            return instance
			
		# check if data type is `HotelGroup`
        if _data_type == "HotelGroup":
            instance.actual_instance = HotelGroup.from_json(json_str)
            return instance
			
		# check if data type is `HotelLengthOfStayCriterion`
        if _data_type == "HotelLengthOfStayCriterion":
            instance.actual_instance = HotelLengthOfStayCriterion.from_json(json_str)
            return instance
			
		# check if data type is `LocationCriterion`
        if _data_type == "LocationCriterion":
            instance.actual_instance = LocationCriterion.from_json(json_str)
            return instance
			
		# check if data type is `LocationIntentCriterion`
        if _data_type == "LocationIntentCriterion":
            instance.actual_instance = LocationIntentCriterion.from_json(json_str)
            return instance
			
		# check if data type is `PlacementCriterion`
        if _data_type == "PlacementCriterion":
            instance.actual_instance = PlacementCriterion.from_json(json_str)
            return instance
			
		# check if data type is `ProductPartition`
        if _data_type == "ProductPartition":
            instance.actual_instance = ProductPartition.from_json(json_str)
            return instance
			
		# check if data type is `ProductScope`
        if _data_type == "ProductScope":
            instance.actual_instance = ProductScope.from_json(json_str)
            return instance
			
		# check if data type is `ProfileCriterion`
        if _data_type == "ProfileCriterion":
            instance.actual_instance = ProfileCriterion.from_json(json_str)
            return instance
			
		# check if data type is `RadiusCriterion`
        if _data_type == "RadiusCriterion":
            instance.actual_instance = RadiusCriterion.from_json(json_str)
            return instance
			
		# check if data type is `StoreCriterion`
        if _data_type == "StoreCriterion":
            instance.actual_instance = StoreCriterion.from_json(json_str)
            return instance
			
		# check if data type is `Webpage`
        if _data_type == "Webpage":
            instance.actual_instance = Webpage.from_json(json_str)
            return instance
			

        # deserialize data into AgeCriterion
        try:
            instance.actual_instance = AgeCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into AudienceCriterion
        try:
            instance.actual_instance = AudienceCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into DayTimeCriterion
        try:
            instance.actual_instance = DayTimeCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into DealCriterion
        try:
            instance.actual_instance = DealCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into DeviceCriterion
        try:
            instance.actual_instance = DeviceCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into GenderCriterion
        try:
            instance.actual_instance = GenderCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into GenreCriterion
        try:
            instance.actual_instance = GenreCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into HotelAdvanceBookingWindowCriterion
        try:
            instance.actual_instance = HotelAdvanceBookingWindowCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into HotelCheckInDateCriterion
        try:
            instance.actual_instance = HotelCheckInDateCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into HotelCheckInDayCriterion
        try:
            instance.actual_instance = HotelCheckInDayCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into HotelDateSelectionTypeCriterion
        try:
            instance.actual_instance = HotelDateSelectionTypeCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into HotelGroup
        try:
            instance.actual_instance = HotelGroup.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into HotelLengthOfStayCriterion
        try:
            instance.actual_instance = HotelLengthOfStayCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into LocationCriterion
        try:
            instance.actual_instance = LocationCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into LocationIntentCriterion
        try:
            instance.actual_instance = LocationIntentCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into PlacementCriterion
        try:
            instance.actual_instance = PlacementCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ProductPartition
        try:
            instance.actual_instance = ProductPartition.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ProductScope
        try:
            instance.actual_instance = ProductScope.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ProfileCriterion
        try:
            instance.actual_instance = ProfileCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into RadiusCriterion
        try:
            instance.actual_instance = RadiusCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into StoreCriterion
        try:
            instance.actual_instance = StoreCriterion.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into Webpage
        try:
            instance.actual_instance = Webpage.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into Criterion with oneOf schemas: AgeCriterion, AudienceCriterion, DayTimeCriterion, DealCriterion, DeviceCriterion, GenderCriterion, GenreCriterion, HotelAdvanceBookingWindowCriterion, HotelCheckInDateCriterion, HotelCheckInDayCriterion, HotelDateSelectionTypeCriterion, HotelGroup, HotelLengthOfStayCriterion, LocationCriterion, LocationIntentCriterion, PlacementCriterion, ProductPartition, ProductScope, ProfileCriterion, RadiusCriterion, StoreCriterion, Webpage. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into Criterion with oneOf schemas: AgeCriterion, AudienceCriterion, DayTimeCriterion, DealCriterion, DeviceCriterion, GenderCriterion, GenreCriterion, HotelAdvanceBookingWindowCriterion, HotelCheckInDateCriterion, HotelCheckInDayCriterion, HotelDateSelectionTypeCriterion, HotelGroup, HotelLengthOfStayCriterion, LocationCriterion, LocationIntentCriterion, PlacementCriterion, ProductPartition, ProductScope, ProfileCriterion, RadiusCriterion, StoreCriterion, Webpage. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], AgeCriterion, AudienceCriterion, DayTimeCriterion, DealCriterion, DeviceCriterion, GenderCriterion, GenreCriterion, HotelAdvanceBookingWindowCriterion, HotelCheckInDateCriterion, HotelCheckInDayCriterion, HotelDateSelectionTypeCriterion, HotelGroup, HotelLengthOfStayCriterion, LocationCriterion, LocationIntentCriterion, PlacementCriterion, ProductPartition, ProductScope, ProfileCriterion, RadiusCriterion, StoreCriterion, Webpage]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())

    def __getattr__(self, name):
        """Forward attribute access to actual_instance"""
        if self.actual_instance is not None and hasattr(self.actual_instance, name):
            return getattr(self.actual_instance, name)
        raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")

    def __setattr__(self, name, value):
        """Forward attribute setting to actual_instance"""
        if name in ['actual_instance', 'oneof_schema_age_criterion_validator', 'oneof_schema_audience_criterion_validator', 'oneof_schema_day_time_criterion_validator', 'oneof_schema_deal_criterion_validator', 'oneof_schema_device_criterion_validator', 'oneof_schema_gender_criterion_validator', 'oneof_schema_genre_criterion_validator', 'oneof_schema_hotel_advance_booking_window_criterion_validator', 'oneof_schema_hotel_check_in_date_criterion_validator', 'oneof_schema_hotel_check_in_day_criterion_validator', 'oneof_schema_hotel_date_selection_type_criterion_validator', 'oneof_schema_hotel_group_validator', 'oneof_schema_hotel_length_of_stay_criterion_validator', 'oneof_schema_location_criterion_validator', 'oneof_schema_location_intent_criterion_validator', 'oneof_schema_placement_criterion_validator', 'oneof_schema_product_partition_validator', 'oneof_schema_product_scope_validator', 'oneof_schema_profile_criterion_validator', 'oneof_schema_radius_criterion_validator', 'oneof_schema_store_criterion_validator', 'oneof_schema_webpage_validator', 'one_of_schemas', 'model_config', 'discriminator_value_class_map']:
            super().__setattr__(name, value)
        elif self.actual_instance is not None and hasattr(self.actual_instance, name):
            setattr(self.actual_instance, name, value)
        else:
            super().__setattr__(name, value)



from bingads.v13.internal.bulk.entities.bulk_entity_identifier import _BulkEntityIdentifier
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.extensions import *
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
Date = _CAMPAIGN_OBJECT_FACTORY_V13.create('Date')

class _BulkAdExtensionBase(_SingleRecordBulkEntity):
    """ This class provides properties that are shared by all bulk ad extension classes.

    *See also:*

    * :class:`.BulkCallAdExtension`
    * :class:`.BulkImageAdExtension`
    * :class:`.BulkLocationAdExtension`
    * :class:`.BulkSiteLinkAdExtension`
    """

    def __init__(self, account_id=None, ad_extension=None):
        super(_BulkAdExtensionBase, self).__init__()

        self._account_id = account_id
        self._ad_extension = ad_extension

    @property
    def account_id(self):
        """ The ad extension's parent account identifier.

        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: int
        """

        return self._account_id

    @account_id.setter
    def account_id(self, account_id):
        self._account_id = account_id

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c._ad_extension.Status),
            csv_to_field=lambda c, v: csv_to_field_enum(c._ad_extension, v, 'Status', AdExtensionStatus)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c._ad_extension.Id),
            csv_to_field=lambda c, v: setattr(c._ad_extension, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Version,
            field_to_csv=lambda c: bulk_str(c._ad_extension.Version),
            csv_to_field=lambda c, v: setattr(c._ad_extension, 'Version', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.account_id),
            csv_to_field=lambda c, v: setattr(c, 'account_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.StartDate,
            field_to_csv=lambda c: field_to_csv_SchedulingDate(c._ad_extension.Scheduling.StartDate, c._ad_extension.Id) if c._ad_extension.Scheduling else None,
            csv_to_field=lambda c, v: csv_to_field_Date(c._ad_extension.Scheduling, 'StartDate', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.EndDate,
            field_to_csv=lambda c: field_to_csv_SchedulingDate(c._ad_extension.Scheduling.EndDate, c._ad_extension.Id) if c._ad_extension.Scheduling else None,
            csv_to_field = lambda c, v: csv_to_field_Date(c._ad_extension.Scheduling, 'EndDate', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AdSchedule,
            field_to_csv=lambda c: field_to_csv_AdSchedule(c._ad_extension.Scheduling, c._ad_extension.Id),
            csv_to_field=lambda c, v: csv_to_field_AdSchedule(c._ad_extension.Scheduling, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.UseSearcherTimeZone,
            field_to_csv=lambda c: field_to_csv_UseSearcherTimeZone(c._ad_extension.Scheduling.UseSearcherTimeZone, c._ad_extension.Id) if c._ad_extension.Scheduling else None,
            csv_to_field=lambda c, v: setattr(c._ad_extension.Scheduling, 'UseSearcherTimeZone', parse_bool(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.DevicePreference,
            field_to_csv=lambda c: bulk_device_preference_str(c._ad_extension.DevicePreference),
            csv_to_field=lambda c, v: setattr(c._ad_extension, 'DevicePreference', parse_device_preference(v))
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        row_values.convert_to_entity(self, _BulkAdExtensionBase._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self.convert_to_values(row_values, _BulkAdExtensionBase._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(_BulkAdExtensionBase, self).read_additional_data(stream_reader)


class _BulkAdExtensionAssociation(_SingleRecordBulkEntity):
    """ This class provides properties that are shared by all bulk ad extension association classes.

    For more information, see Bulk File Schema at https://go.microsoft.com/fwlink/?linkid=846127.
    """

    def __init__(self,
                 ad_extension_id_to_entity_id_association=None,
                 status=None,
                 editorial_status=None):
        super(_BulkAdExtensionAssociation, self).__init__()

        self._status = status
        self._ad_extension_id_to_entity_id_association = ad_extension_id_to_entity_id_association
        self._editorial_status = editorial_status
        self._performance_data = None

    @property
    def ad_extension_id_to_entity_id_association(self):
        """ Defines an association relationship between an ad extension and a supported entity, for example a campaign or ad group.

        :rtype: AdExtensionIdToEntityIdAssociation
        """

        return self._ad_extension_id_to_entity_id_association

    @ad_extension_id_to_entity_id_association.setter
    def ad_extension_id_to_entity_id_association(self, value):
        self._ad_extension_id_to_entity_id_association = value

    @property
    def status(self):
        """ The status of the ad extension association.

        The value is Active if the EntityId and AdExtensionId are associated. The value is Deleted if the association is removed.
        Corresponds to the 'Status' field in the bulk file.

        :rtype: str
        """

        return self._status

    @status.setter
    def status(self, status):
        self._status = status

    @property
    def editorial_status(self):
        """ The editorial status of the ad extension and associated entity.

        For more information, see AdExtensionEditorialStatus at https://go.microsoft.com/fwlink/?linkid=846127.
        Corresponds to the 'Editorial Status' field in the bulk file.

        :rtype: str
        """

        return self._editorial_status

    @editorial_status.setter
    def editorial_status(self, editorial_status):
        self._editorial_status = editorial_status

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.ad_extension_id_to_entity_id_association.AdExtensionId),
            csv_to_field=lambda c, v: setattr(c.ad_extension_id_to_entity_id_association, 'AdExtensionId', int(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.status),
            csv_to_field=lambda c, v: setattr(c, '_status', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.ad_extension_id_to_entity_id_association.EntityId),
            csv_to_field=lambda c, v: setattr(c.ad_extension_id_to_entity_id_association, 'EntityId', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.EditorialStatus,
            field_to_csv=lambda c: c.editorial_status,
            csv_to_field=lambda c, v: setattr(c, '_editorial_status', v if v else None)
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        self._ad_extension_id_to_entity_id_association = _CAMPAIGN_OBJECT_FACTORY_V13.create('AdExtensionIdToEntityIdAssociation')
        row_values.convert_to_entity(self, _BulkAdExtensionAssociation._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(
            self._ad_extension_id_to_entity_id_association,
            'ad_extension_id_to_entity_id_association'
        )
        self.convert_to_values(row_values, _BulkAdExtensionAssociation._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(_BulkAdExtensionAssociation, self).read_additional_data(stream_reader)


class _BulkAccountAdExtensionAssociation(_BulkAdExtensionAssociation):
    """ This abstract class provides properties that are shared by all bulk account ad extension association classes. """

    def __init__(self,
                 ad_extension_id_to_entity_id_association=None,
                 status=None,
                 editorial_status=None):
        super(_BulkAccountAdExtensionAssociation, self).__init__(
            ad_extension_id_to_entity_id_association,
            status,
            editorial_status,
        )


class _BulkCampaignAdExtensionAssociation(_BulkAdExtensionAssociation):
    """ This abstract class provides properties that are shared by all bulk campaign ad extension association classes. """

    def __init__(self,
                 ad_extension_id_to_entity_id_association=None,
                 status=None,
                 editorial_status=None):
        super(_BulkCampaignAdExtensionAssociation, self).__init__(
            ad_extension_id_to_entity_id_association,
            status,
            editorial_status,
        )
        self._campaign_name = None

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Campaign,
            field_to_csv=lambda c: c.campaign_name,
            csv_to_field=lambda c, v: setattr(c, '_campaign_name', v)
        )
    ]

    def process_mappings_from_row_values(self, row_values):
        super(_BulkCampaignAdExtensionAssociation, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, _BulkCampaignAdExtensionAssociation._MAPPINGS)

    @property
    def campaign_name(self):
        """ The name of the campaign containing the ad group that the ad extension is associated.

        Corresponds to the 'Campaign' field in the bulk file.

        :rtype: str
        """

        return self._campaign_name


class _BulkAdGroupAdExtensionAssociation(_BulkAdExtensionAssociation):
    """ This abstract class provides properties that are shared by all bulk ad group ad extension association classes. """

    def __init__(self,
                 ad_extension_id_to_entity_id_association=None,
                 status=None,
                 editorial_status=None):
        super(_BulkAdGroupAdExtensionAssociation, self).__init__(
            ad_extension_id_to_entity_id_association,
            status,
            editorial_status,
        )
        self._ad_group_name = None
        self._campaign_name = None

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.AdGroup,
            field_to_csv=lambda c: c.ad_group_name,
            csv_to_field=lambda c, v: setattr(c, '_ad_group_name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Campaign,
            field_to_csv=lambda c: c.campaign_name,
            csv_to_field=lambda c, v: setattr(c, '_campaign_name', v)
        )
    ]

    def process_mappings_from_row_values(self, row_values):
        super(_BulkAdGroupAdExtensionAssociation, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, _BulkAdGroupAdExtensionAssociation._MAPPINGS)

    @property
    def ad_group_name(self):
        """ The name of the ad group that the ad extension is associated.

        Corresponds to the 'AdGroup' field in the bulk file.

        :rtype str
        """

        return self._ad_group_name

    @property
    def campaign_name(self):
        """ The name of the campaign containing the ad group that the ad extension is associated.

        Corresponds to the 'Campaign' field in the bulk file.

        :rtype: str
        """

        return self._campaign_name


class _BulkAdExtensionIdentifier(_BulkEntityIdentifier):
    def __init__(self,
                 account_id=None,
                 ad_extension_id=None,
                 status=None,
                 version=None):
        super(_BulkAdExtensionIdentifier, self).__init__()

        self._account_id = account_id
        self._ad_extension_id = ad_extension_id
        self._status = status
        self._version = version

    @property
    def account_id(self):
        return self._account_id

    @account_id.setter
    def account_id(self, account_id):
        self._account_id = account_id

    @property
    def ad_extension_id(self):
        return self._ad_extension_id

    @ad_extension_id.setter
    def ad_extension_id(self, ad_extension_id):
        self._ad_extension_id = ad_extension_id

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, status):
        self._status = status

    @property
    def version(self):
        return self._version

    @version.setter
    def version(self, version):
        self._version = version

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.status),
            csv_to_field=lambda c, v: setattr(c, 'status', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.ad_extension_id),
            csv_to_field=lambda c, v: setattr(c, 'ad_extension_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Version,
            field_to_csv=lambda c: bulk_str(c.version),
            csv_to_field=lambda c, v: setattr(c, 'version', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.account_id),
            csv_to_field=lambda c, v: setattr(c, 'account_id', int(v) if v else None)
        ),
    ]

    def read_from_row_values(self, row_values):
        row_values.convert_to_entity(self, _BulkAdExtensionIdentifier._MAPPINGS)

    def write_to_row_values(self, row_values, exclude_readonly_data):
        self.convert_to_values(row_values, _BulkAdExtensionIdentifier._MAPPINGS)

    def __eq__(self, other):
        raise NotImplementedError()

    @property
    def is_delete_row(self):
        return self._status == 'Deleted'



from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from .common import *
from .common import _BulkAdExtensionBase
from .common import _BulkCampaignAdExtensionAssociation
from .common import _BulkAccountAdExtensionAssociation

_LocationAdExtension = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('LocationAdExtension'))


class BulkLocationAdExtension(_BulkAdExtensionBase):
    """ Represents an location ad extension.

    This class exposes the :attr:`location_ad_extension` property that can be read and written
    as fields of the Location Ad Extension record in a bulk file.

    For more information, see Location Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, account_id=None, ad_extension=None):
        if ad_extension and not isinstance(ad_extension, _LocationAdExtension):
            raise ValueError('The type of ad_extension is: {0}, should be: {1}'.format(
                type(ad_extension),
                'LocationAdExtension'
            ))
        super(BulkLocationAdExtension, self).__init__(
            account_id=account_id,
            ad_extension=ad_extension
        )

    @property
    def location_ad_extension(self):
        """ The location ad extension.

        see Location Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.
        """

        return self._ad_extension

    @location_ad_extension.setter
    def location_ad_extension(self, value):
        self._ad_extension = value

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.BusinessName,
            field_to_csv=lambda c: c.location_ad_extension.CompanyName,
            csv_to_field=lambda c, v: setattr(c.location_ad_extension, 'CompanyName', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.PhoneNumber,
            field_to_csv=lambda c: bulk_optional_str(c.location_ad_extension.PhoneNumber, c.location_ad_extension.Id),
            csv_to_field=lambda c, v: setattr(c.location_ad_extension, 'PhoneNumber', v if v else '')
        ),
        _SimpleBulkMapping(
            header=_StringTable.GeoCodeStatus,
            field_to_csv=lambda c: bulk_str(c.location_ad_extension.GeoCodeStatus),
            csv_to_field=lambda c, v: csv_to_field_enum(c.location_ad_extension, v, 'GeoCodeStatus', BusinessGeoCodeStatus)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AddressLine1,
            field_to_csv=lambda c: BulkLocationAdExtension.get_address_part(
                c,
                lambda x: x.StreetAddress
            ),
            csv_to_field=lambda c, v: BulkLocationAdExtension.set_address_part(
                c,
                lambda x: setattr(x, 'StreetAddress', v)
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.AddressLine2,
            field_to_csv=lambda c: BulkLocationAdExtension.get_address_part(
                c,
                lambda x: bulk_optional_str(x.StreetAddress2, c.location_ad_extension.Id)
            ),
            csv_to_field=lambda c, v: BulkLocationAdExtension.set_address_part(
                c,
                lambda x: setattr(x, 'StreetAddress2', v if v else '')
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.City,
            field_to_csv=lambda c: BulkLocationAdExtension.get_address_part(c, lambda x: x.CityName),
            csv_to_field=lambda c, v: BulkLocationAdExtension.set_address_part(
                c,
                lambda x: setattr(x, 'CityName', v)
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.ProvinceName,
            field_to_csv=lambda c: BulkLocationAdExtension.get_address_part(c, lambda x: x.ProvinceName),
            csv_to_field=lambda c, v: BulkLocationAdExtension.set_address_part(
                c,
                lambda x: setattr(x, 'ProvinceName', v)
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.StateOrProvince,
            field_to_csv=lambda c: BulkLocationAdExtension.get_address_part(c, lambda x: x.ProvinceCode),
            csv_to_field=lambda c, v: BulkLocationAdExtension.set_address_part(
                c,
                lambda x: setattr(x, 'ProvinceCode', v)
            )
        ),

        _SimpleBulkMapping(
            header=_StringTable.PostalCode,
            field_to_csv=lambda c: BulkLocationAdExtension.get_address_part(c, lambda x: x.PostalCode),
            csv_to_field=lambda c, v: BulkLocationAdExtension.set_address_part(
                c,
                lambda x: setattr(x, 'PostalCode', v)
            )
        ),

        _SimpleBulkMapping(
            header=_StringTable.CountryCode,
            field_to_csv=lambda c: BulkLocationAdExtension.get_address_part(c, lambda x: x.CountryCode),
            csv_to_field=lambda c, v: BulkLocationAdExtension.set_address_part(
                c,
                lambda x: setattr(x, 'CountryCode', v)
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.Latitude,
            field_to_csv=lambda c: BulkLocationAdExtension.get_geo_point_part(
                c,
                lambda x: bulk_str(
                    float(x.LatitudeInMicroDegrees) / 1000000.0
                )
                if x.LatitudeInMicroDegrees is not None else None
            ),
            csv_to_field=lambda c, v: BulkLocationAdExtension.set_geo_point_part(
                c,
                lambda x, latitude: setattr(x, 'LatitudeInMicroDegrees', int(round(float(latitude) * 1000000))),
                v
            )
        ),
        _SimpleBulkMapping(
            header=_StringTable.Longitude,
            field_to_csv=lambda c: BulkLocationAdExtension.get_geo_point_part(
                c,
                lambda x: bulk_str(
                    float(x.LongitudeInMicroDegrees) / 1000000.0
                )
                if x.LongitudeInMicroDegrees is not None else None
            ),
            csv_to_field=lambda c, v: BulkLocationAdExtension.set_geo_point_part(
                c,
                lambda x, longitude: setattr(x, 'LongitudeInMicroDegrees', int(round(float(longitude) * 1000000))),
                v
            )
        ),
    ]

    @staticmethod
    def get_address_part(bulk_ad_extension, get_func):
        if bulk_ad_extension.location_ad_extension.Address is not None:
            return get_func(bulk_ad_extension.location_ad_extension.Address)
        else:
            return None

    @staticmethod
    def set_address_part(bulk_ad_extension, set_func):
        if bulk_ad_extension.location_ad_extension.Address is None:
            bulk_ad_extension.location_ad_extension.Address = _CAMPAIGN_OBJECT_FACTORY_V13.create('Address')
        set_func(bulk_ad_extension.location_ad_extension.Address)

    @staticmethod
    def get_geo_point_part(bulk_ad_extension, get_func):
        if bulk_ad_extension.location_ad_extension.GeoPoint is not None:
            return get_func(bulk_ad_extension.location_ad_extension.GeoPoint)
        else:
            return None

    @staticmethod
    def set_geo_point_part(bulk_ad_extension, set_func, value):
        if not value:
            return
        if bulk_ad_extension.location_ad_extension.GeoPoint is None:
            bulk_ad_extension.location_ad_extension.GeoPoint = _CAMPAIGN_OBJECT_FACTORY_V13.create('GeoPoint')
        set_func(bulk_ad_extension.location_ad_extension.GeoPoint, value)

    def process_mappings_from_row_values(self, row_values):
        self.location_ad_extension = _CAMPAIGN_OBJECT_FACTORY_V13.create('LocationAdExtension')
        self.location_ad_extension.Type = 'LocationAdExtension'
        if row_values[_StringTable.Latitude] or row_values[_StringTable.Longitude]:
            self.location_ad_extension.GeoPoint = _CAMPAIGN_OBJECT_FACTORY_V13.create('GeoPoint')
        super(BulkLocationAdExtension, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkLocationAdExtension._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.location_ad_extension, 'location_ad_extension')
        super(BulkLocationAdExtension, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkLocationAdExtension._MAPPINGS)


class BulkAccountLocationAdExtension(_BulkAccountAdExtensionAssociation):
    """ Represents an account level location ad extension.

    This class exposes properties that can be read and written
    as fields of the Account Location Ad Extension record in a bulk file.

    For more information, see Account Location Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass


class BulkCampaignLocationAdExtension(_BulkCampaignAdExtensionAssociation):
    """ Represents a campaign level location ad extension.

    This class exposes properties that can be read and written
    as fields of the Campaign Location Ad Extension record in a bulk file.

    For more information, see Campaign Location Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass


# coding: utf-8

"""
    CustomerManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.customer.account_life_cycle_status import AccountLifeCycleStatus
from typing_extensions import Self

class AccountInfoWithCustomerData(BaseModel):
    """
    AccountInfoWithCustomerData
    """ # noqa: E501
    customer_id: Optional[StrictStr] = Field(default=None, alias="CustomerId")
    customer_name: Optional[StrictStr] = Field(default=None, alias="CustomerName")
    account_id: Optional[StrictStr] = Field(default=None, alias="AccountId")
    account_name: Optional[StrictStr] = Field(default=None, alias="AccountName")
    account_number: Optional[StrictStr] = Field(default=None, alias="AccountNumber")
    account_life_cycle_status: Optional[AccountLifeCycleStatus] = Field(default=None, alias="AccountLifeCycleStatus")
    pause_reason: Optional[StrictInt] = Field(default=None, alias="PauseReason")
    account_mode: Optional[StrictStr] = Field(default=None, alias="AccountMode")
    __properties: ClassVar[List[str]] = ["CustomerId", "CustomerName", "AccountId", "AccountName", "AccountNumber", "AccountLifeCycleStatus", "PauseReason", "AccountMode"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if customer_id (nullable) is None
        # and model_fields_set contains the field
        if self.customer_id is None and "customer_id" in self.model_fields_set:
            _dict['CustomerId'] = None

        # set to None if customer_name (nullable) is None
        # and model_fields_set contains the field
        if self.customer_name is None and "customer_name" in self.model_fields_set:
            _dict['CustomerName'] = None

        # set to None if account_id (nullable) is None
        # and model_fields_set contains the field
        if self.account_id is None and "account_id" in self.model_fields_set:
            _dict['AccountId'] = None

        # set to None if account_name (nullable) is None
        # and model_fields_set contains the field
        if self.account_name is None and "account_name" in self.model_fields_set:
            _dict['AccountName'] = None

        # set to None if account_number (nullable) is None
        # and model_fields_set contains the field
        if self.account_number is None and "account_number" in self.model_fields_set:
            _dict['AccountNumber'] = None

        # set to None if account_life_cycle_status (nullable) is None
        # and model_fields_set contains the field
        if self.account_life_cycle_status is None and "account_life_cycle_status" in self.model_fields_set:
            _dict['AccountLifeCycleStatus'] = None

        # set to None if pause_reason (nullable) is None
        # and model_fields_set contains the field
        if self.pause_reason is None and "pause_reason" in self.model_fields_set:
            _dict['PauseReason'] = None

        # set to None if account_mode (nullable) is None
        # and model_fields_set contains the field
        if self.account_mode is None and "account_mode" in self.model_fields_set:
            _dict['AccountMode'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AccountInfoWithCustomerData from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "CustomerId": obj.get("CustomerId") if obj.get("CustomerId") is not None else None,
                        "CustomerName": obj.get("CustomerName") if obj.get("CustomerName") is not None else None,
                        "AccountId": obj.get("AccountId") if obj.get("AccountId") is not None else None,
                        "AccountName": obj.get("AccountName") if obj.get("AccountName") is not None else None,
                        "AccountNumber": obj.get("AccountNumber") if obj.get("AccountNumber") is not None else None,
                        "AccountLifeCycleStatus": obj.get("AccountLifeCycleStatus") if obj.get("AccountLifeCycleStatus") is not None else None,
                        "PauseReason": obj.get("PauseReason") if obj.get("PauseReason") is not None else None,
                        "AccountMode": obj.get("AccountMode") if obj.get("AccountMode") is not None else None
        })
        return _obj
